package val.gp2x;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;

import com.synclast.Font;
import com.synclast.FontBitmapped;

import sdljava.SDLException;
import sdljava.video.SDLColor;
import sdljava.video.SDLRect;
import sdljava.video.SDLSurface;
import sdljava.video.SDLVideo;

public class FileSystemNavigator
{
  public static final int EXITCODE_LOAD = 0xE0;
  public static final int EXITCODE_QUIT = 0xE1;
  
  // the big interface
  private GP2x gp2x;
  
  private String descriptorFileName = null;
  private boolean finished = false;
  private int exitCode = EXITCODE_QUIT;

  // graphics related
  private Font fontBig = null;
  private FontBitmapped fontSmall = null;
  
  private SDLColor colorBlue = null;
  private SDLColor colorLightGray = null;
  private SDLColor colorYellow = null;
  private SDLColor colorRed = null;
  
  private SDLSurface appLogo = null;
  private String appName = null;
  
  private MyFileNameChecker myFileNameChecker = null;
  private File location = null;
  private File names[] = null;
  private AppProperties ap = null;
  
  private int selectedItem = 0;
  private int firstVisibleItem = 0;
  private int maxListItems = 10;
  
  
  public void initialize(GP2x gp2x) throws Exception
  {
    this.gp2x = gp2x;

    fontBig = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_LARGE);
    fontSmall = FontBitmapped.getFont(FontBitmapped.FACE_SYSTEM, FontBitmapped.STYLE_NORMAL, 8);
    
    colorBlue = new SDLColor(0, 0, 255);
    colorLightGray = new SDLColor(200, 200, 200);
    colorYellow = new SDLColor(255, 255, 0);
    colorRed = new SDLColor(255, 0, 0);
    
    // init initial list of files
    myFileNameChecker = new MyFileNameChecker();
    location = new File(".");
    updateFileList();
    
    descriptorFileName = null;
    finished = false;
    exitCode = EXITCODE_QUIT;
  }

  private void updateFileList()
  {
    names = location.listFiles(myFileNameChecker);
    if (names == null)
      names = new File[0];
    
    Arrays.sort(names, myFileNameChecker);
    
    if (appLogo != null)
    {
      try {        appLogo.freeSurface();      } catch (SDLException e) { }
      appLogo = null;
    }
    
    appName = null;
    
    for (int i=0; i<names.length; i++)
    {
      if (names[i].getName().equals("launcher.properties"))
      {
        ap = new AppProperties(names[i].getAbsolutePath());
        if (!ap.isValid())
        {
          ap = null;
        }
        else
        {
          if (ap.getAppLogo() != null)
            try {              appLogo = SDLVideo.loadBMP(names[i].getParent() +File.separator+ ap.getAppLogo());            }catch (Exception e) {}
          appName = ap.getAppName();
        }
      }
    }
  }
  
  private void moveToLocation(String newLocation)
  {
    try
    {
      location = new File( location.getCanonicalFile() + File.separator + newLocation);
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }

    updateFileList();
  }
  
  
  public void finish() throws Exception
  {
  }
 
  public int mainLoop()
  {
    try
    {
      while (!finished)
      {
        Thread.sleep(100); // We repaint the screen every 100 milliseconds (about 10 fps)
        events();
        sounds();
        update();
        repaint();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return EXITCODE_QUIT;
    }

    return exitCode;
  }
  
  
  /**
   * Play any environmental sound
   */
  private void sounds()
  {
  }


  private void syncVisibleWindow()
  {
    if (selectedItem < 0)
      selectedItem = names.length - 1;
    if (selectedItem >= names.length)
      selectedItem = 0;

    if (selectedItem < firstVisibleItem)
      firstVisibleItem = selectedItem;
    if (selectedItem >= firstVisibleItem+maxListItems)
      firstVisibleItem = selectedItem;
  }
  
  /**
   * Performs actions depending on pressed/clicked buttons
   */
  public void events() throws Exception
  {
    gp2x.getJoyButtonChanges();

    if (gp2x.isButtonPressed(GP2x.VK_UP)) {
      selectedItem--;
      syncVisibleWindow();
    }
    if (gp2x.isButtonPressed(GP2x.VK_DOWN)) {
      selectedItem++;
      syncVisibleWindow();
    }
    if (gp2x.isButtonPressed(GP2x.VK_LEFT)) {
    }
    if (gp2x.isButtonPressed(GP2x.VK_RIGHT)) {
    }

    if (gp2x.isButtonPressed(GP2x.VK_UP_LEFT)) {
    }
    if (gp2x.isButtonPressed(GP2x.VK_UP_RIGHT)) {
    }
    if (gp2x.isButtonPressed(GP2x.VK_DOWN_LEFT)) {
    }
    if (gp2x.isButtonPressed(GP2x.VK_DOWN_RIGHT)) {
    }

    // upper directory
    if (gp2x.isButtonClicked(GP2x.VK_FY)) {
      moveToLocation("..");
      selectedItem = firstVisibleItem = 0;
      syncVisibleWindow();
    }
    
    // QUIT
    if (gp2x.isButtonClicked(GP2x.VK_FX)) {
      exitCode = EXITCODE_QUIT;
      finished = true;
    }
    // OPTIONS
    if (gp2x.isButtonClicked(GP2x.VK_FA)) {
    }
    // OK start game
    if (gp2x.isButtonClicked(GP2x.VK_FB)) {
      if (selectedItem < names.length)
      {
        if (names[selectedItem].isDirectory())
        {
          moveToLocation(names[selectedItem].getName());
          selectedItem = firstVisibleItem = 0;
          syncVisibleWindow();
        }
        else
        {
          setDescriptorFileName(names[selectedItem].getCanonicalPath());
          exitCode = EXITCODE_LOAD;
          finished = true;
        }
      }
    }
        
  }
  

  /**
   * Redraws completely the screen
   */
  private void writeStringAt(Font font, String s, int x, int y, SDLColor color)
  {
    try
    {
      SDLSurface s1 = font.writeString(s, color);
      gp2x.drawImage(s1, x, y);
      s1.freeSurface();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
  
  /**
   * Redraws completely the screen
   */
  private void writeStringAt(FontBitmapped font, String s, int x, int y, SDLColor color)
  {
    try
    {
      SDLSurface s1 = font.writeString(s, color);
      gp2x.drawImage(s1, x, y);
      s1.freeSurface();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
  
  public void repaint()
  {
    gp2x.clearScreen();
    
    writeStringAt(fontBig, "GP2X Java App Launcher", 2, 2, colorBlue);

    writeStringAt(fontSmall, "Y: upper dir   X: quit   B:down dir/launch", 2, 220, colorBlue);
    
    if (location.isDirectory())
    {
      try
      {
        writeStringAt(fontSmall, location.getCanonicalPath(), 2, 22, colorYellow);
      }
      catch (Exception e) {
        writeStringAt(fontSmall, "<<inaccessible>>", 2, 22, colorYellow);
        e.printStackTrace();
      }

      if (appName != null)
        writeStringAt(fontSmall, appName, 160, 40, colorRed);
      if (appLogo != null)
        try{ appLogo.blitSurface(gp2x.getFramebuffer(), new SDLRect(200,60,appLogo.getWidth(),appLogo.getHeight())); } catch (Exception e) { }

      int starty = 56;
      for (int i=firstVisibleItem; i<firstVisibleItem+maxListItems; i++)
      {
        if (i >= names.length)
          break;
        
        if (names[i].isFile())
          writeStringAt(fontSmall, names[i].getName(), 20, starty, colorLightGray);
        else
          writeStringAt(fontSmall, names[i].getName() + File.separator, 20, starty, colorLightGray);

        if (i == selectedItem)
          writeStringAt(fontSmall, ">", 5, starty, colorRed);
          
        starty += 16;
      }
    }
    
    gp2x.repaint();
  }

  
  /**
   * Update any animation on the screen
   */
  public void update()
  {
  }
  
  
  
  public String getDescriptorFileName()
  {
    return descriptorFileName;
  }

  public void setDescriptorFileName(String jarFileName)
  {
    this.descriptorFileName = jarFileName;
  }
  
}

