package val.gp2x;

import sdljava.SDLException;
import sdljava.SDLMain;
import sdljava.SDLVersion;
import sdljava.audio.SDLAudio;
import sdljava.event.SDLEvent;
import sdljava.event.SDLEventState;
import sdljava.event.SDLJoyAxisEvent;
import sdljava.event.SDLJoyButtonEvent;
import sdljava.event.SDLKeyboardEvent;
import sdljava.joystick.SDLJoystick;
import sdljava.mixer.MixerSpec;
import sdljava.mixer.SDLMixer;
import sdljava.ttf.SDLTTF;
import sdljava.video.SDLRect;
import sdljava.video.SDLSurface;
import sdljava.video.SDLVideo;
import sdljava.x.swig.SDLKeyValues;
import sdljava.x.swig.SDLPressedState;
import sdljava.x.swig.SWIG_SDLEvent;

//*************************************************************************************
//**********************   Our Canvas  ************************************************
//*************************************************************************************
public class GP2x
{
  public static final int  VK_NONE       = -1 ;

  public static final int  VK_UP         = 0 ;
  public static final int  VK_UP_LEFT    = 1 ;
  public static final int  VK_LEFT       = 2 ;
  public static final int  VK_DOWN_LEFT  = 3 ;
  public static final int  VK_DOWN       = 4 ;
  public static final int  VK_DOWN_RIGHT = 5 ;
  public static final int  VK_RIGHT      = 6 ;
  public static final int  VK_UP_RIGHT   = 7 ;
  public static final int  VK_START      = 8 ;
  public static final int  VK_SELECT     = 9 ;
  public static final int  VK_FL         = 10 ;
  public static final int  VK_FR         = 11 ;
  public static final int  VK_FA         = 12 ;
  public static final int  VK_FB         = 13 ;
  public static final int  VK_FX         = 14 ;
  public static final int  VK_FY         = 15 ;
  public static final int  VK_VOL_UP     = 16 ;
  public static final int  VK_VOL_DOWN   = 17 ;
  public static final int  VK_TAT        = 18 ;

  /*
   * Maps hardware buttons => symbolic name buttons
   * on GP2X they are mapped 1-1
   * on Darwin/MacOSX they are re-mapped for the xtreme mini-joystick, that lacks 4 directions
   */
  public static final String GP2XJoyButtonsNames[] = new String[]
      { "VK_UP", "VK_UP_LEFT", "VK_LEFT", "VK_DOWN_LEFT", "VK_DOWN", "VK_DOWN_RIGHT", "VK_RIGHT", "VK_UP_RIGHT", "VK_START", "VK_SELECT",
        "VK_FL", "VK_FR", "VK_FA", "VK_FB", "VK_FX", "VK_FY", "VK_VOL_UP", "VK_VOL_DOWN", "VK_TAT" };

  public static final String MacOSXJoyButtonsNames[] = new String[]
      { "VK_FA", "VK_FB", "VK_VOL_UP", "VK_FX", "VK_FY", "VK_VOL_DOWN", "VK_FL", "VK_FR", "VK_START", "VK_SELECT",
        "VK_UP", "VK_UP_LEFT", "VK_LEFT", "VK_DOWN_LEFT", "VK_DOWN", "VK_DOWN_RIGHT", "VK_RIGHT", "VK_UP_RIGHT", "VK_TAT" };

  public static final int  MacOSXJoyButtons[] = new int[]
      { VK_FA, VK_FB, VK_VOL_UP, VK_FX, VK_FY, VK_VOL_DOWN, VK_FL, VK_FR, VK_START, VK_SELECT,
        VK_UP, VK_UP_LEFT, VK_LEFT, VK_DOWN_LEFT, VK_DOWN, VK_DOWN_RIGHT, VK_RIGHT, VK_UP_RIGHT,
        VK_TAT };
                                                
  public static final int  GP2XJoyButtons[] = new int[]
      { VK_UP, VK_UP_LEFT, VK_LEFT, VK_DOWN_LEFT, VK_DOWN, VK_DOWN_RIGHT, VK_RIGHT, VK_UP_RIGHT, VK_START, VK_SELECT,
        VK_FL, VK_FR, VK_FA, VK_FB, VK_FX, VK_FY, VK_VOL_UP, VK_VOL_DOWN, VK_TAT };
                                 
  public static int JoyButtons[];
  public static String JoyButtonsNames[];
  {
    String systemName = System.getProperty("os.name").toLowerCase();
    System.out.println("System is "+systemName+"...");
    
    if (systemName.startsWith("darwin"))
    {
      System.out.println("Setting up for Darwin...");
      
      JoyButtons = MacOSXJoyButtons;
      JoyButtonsNames = MacOSXJoyButtonsNames;
    }
    else if (systemName.startsWith("windows"))
    {
      System.out.println("Setting up for Windows...");
      
      JoyButtons = GP2XJoyButtons;
      JoyButtonsNames = GP2XJoyButtonsNames;
    }
    else
    {
      System.out.println("Setting up for GP2X...");
      
      JoyButtons = GP2XJoyButtons;
      JoyButtonsNames = GP2XJoyButtonsNames;
    }
  }
  
  private boolean joyButtonPressedStates[];
  private boolean joyButtonClickedStates[];
  
  private SDLSurface framebuffer = null;
  private SDLSurface backbuffer = null;
  private SDLJoystick joystick = null;

  public GP2x() throws Exception
  {
    this.init();
  }

  private void init() throws SDLException
  {
    SDLMain.init(SDLMain.SDL_INIT_VIDEO | SDLMain.SDL_INIT_JOYSTICK | SDLMain.SDL_INIT_AUDIO | SDLMain.SDL_INIT_NOPARACHUTE);
    SDLTTF.init();

    SDLVersion v = SDLMain.getSDLVersion();
    System.out.println("SDL subsystem initialized: " + v.getMajor() + "." + v.getMinor() + "." + v.getPatch());

    // framebuffer = SDLVideo.setVideoMode(320, 240, 32, (long)SDLVideo.SDL_HWSURFACE|SDLVideo.SDL_DOUBLEBUF|SDLVideo.SDL_RESIZABLE);
    framebuffer = SDLVideo.setVideoMode(320, 240, 16, (long) SDLVideo.SDL_HWSURFACE);
    SWIG_SDLEvent.SDL_ShowCursor(0);
    backbuffer = SDLVideo.createRGBSurface(SDLVideo.SDL_SWSURFACE, framebuffer.getWidth(), framebuffer.getHeight(),
        framebuffer.getFormat().getBitsPerPixel(),
        framebuffer.getFormat().getRMask(),
        framebuffer.getFormat().getGMask(),
        framebuffer.getFormat().getBMask(),
        framebuffer.getFormat().getAMask()
        );

    // videoDriverName causes crash right now...not sure why
    // System.out.println("Video Driver=" + SDLVideo.videoDriverName());
    System.out.println("framebuffer=" + framebuffer);
    System.out.println("backbuffer=" + backbuffer);

    try
    {
      if (SDLJoystick.numJoysticks() > 0)
      {
        SDLEvent.joystickEventState(SDLEventState.ENABLE);
        joystick = SDLJoystick.joystickOpen(0);
        // System.out.println("joystick Driver=" + SDLJoystick.joystickName(0));
        System.out.println("joystick=" + joystick);
      }
      else
      {
        System.out.println("No joystick available");
      }
    }
    catch (Exception e)
    {
      // no Joystick
      System.out.println("Exception initializing joystick");
      e.printStackTrace();
    }
    
    joyButtonPressedStates = new boolean[JoyButtons.length];
    joyButtonClickedStates = new boolean[JoyButtons.length];
    for (int x=0; x<JoyButtons.length; x++)
      joyButtonClickedStates[x] = joyButtonPressedStates[x] = false;
    
    SDLMixer.openAudio(11050, SDLAudio.AUDIO_S16LSB, 2, 64);
    MixerSpec spec = SDLMixer.querySpec();
    System.out.println(spec);
    int result = SDLMixer.allocateChannels(16);
    System.out.println("Mixer: allocated "+result+" channels");
  }

  public void destroy()
  {
    try
    {
      SDLMixer.close();
      if (joystick != null)
        joystick.joystickClose();
      SDLTTF.quit();
      SDLMain.quit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * update the button map state
   * @return true on any change
   * @throws Exception
   */
  public boolean getJoyButtonChanges() throws Exception
  {
    SDLEvent e = SDLEvent.pollEvent();

    if (e == null)
      return false;
    
    if (e instanceof SDLKeyboardEvent)
    {
      SDLKeyboardEvent kbe = (SDLKeyboardEvent)e;
      int keySym = kbe.getSym();
      int button = 0;
      SDLKeyValues swigValue = SDLKeyValues.swigToEnum(keySym);

      if (swigValue == SDLKeyValues.SDLK_UP) button = VK_UP;
      else if (swigValue == SDLKeyValues.SDLK_DOWN) button = VK_DOWN;
      else if (swigValue == SDLKeyValues.SDLK_LEFT) button = VK_LEFT;
      else if (swigValue == SDLKeyValues.SDLK_RIGHT) button = VK_RIGHT;
      else if (swigValue == SDLKeyValues.SDLK_a) button = VK_FA;
      else if (swigValue == SDLKeyValues.SDLK_b) button = VK_FB;
      else if (swigValue == SDLKeyValues.SDLK_x) button = VK_FX;
      else if (swigValue == SDLKeyValues.SDLK_y) button = VK_FY;
      else if (swigValue == SDLKeyValues.SDLK_RETURN) button = VK_START;
      else if (swigValue == SDLKeyValues.SDLK_BACKSPACE) button = VK_SELECT;
      else if (swigValue == SDLKeyValues.SDLK_l) button = VK_FL;
      else if (swigValue == SDLKeyValues.SDLK_r) button = VK_FR;
      else if (swigValue == SDLKeyValues.SDLK_PAGEDOWN) button = VK_VOL_DOWN;
      else if (swigValue == SDLKeyValues.SDLK_PAGEUP) button = VK_VOL_UP;
      else return false;

      if (kbe.getState() == SDLPressedState.PRESSED)
      {
        joyButtonPressedStates[JoyButtons[button]] = true;
        joyButtonClickedStates[JoyButtons[button]] = false;
      }
      else
      {
        joyButtonPressedStates[JoyButtons[button]] = false;
        joyButtonClickedStates[JoyButtons[button]] = true;
      }
        
      return true;
    }
    else if ( e instanceof SDLJoyButtonEvent)
    {
      SDLJoyButtonEvent jbe = (SDLJoyButtonEvent)e;
      int button = jbe.getButton();
      
      // System.out.println("Event:[" + jbe + "]");
      // System.out.println("Joy Map: "+ JoyButtonsNames[button]);
      
      if (jbe.getState() == SDLPressedState.PRESSED)
      {
        joyButtonPressedStates[JoyButtons[button]] = true;
        joyButtonClickedStates[JoyButtons[button]] = false;
      }
      else
      {
        joyButtonPressedStates[JoyButtons[button]] = false;
        joyButtonClickedStates[JoyButtons[button]] = true;
      }
        
      return true;
    }
    else if (e instanceof SDLJoyAxisEvent)
    {
      // GP2X emulation for JoyStick with Axis
      /*
       *            Asse1 -32K
       *               |
       *  Asse0 -32K --+-- Asse0 +32K
       *               |
       *            Asse1 +32K
       */
      SDLJoyAxisEvent jae = (SDLJoyAxisEvent)e;
      // System.out.println("Event:[" + jae + "]");
      
      int axis = jae.getAxis();
      int value = jae.getValue();
      
      if (axis == 1) // up-down
      {
        if (value < -32000) // up pressed
        {
          joyButtonPressedStates[VK_UP] = true;
          joyButtonClickedStates[VK_UP] = false;
        }
        else if (value > 32000) // down pressed
        {
          joyButtonPressedStates[VK_DOWN] = true;
          joyButtonClickedStates[VK_DOWN] = false;
        }
        else
        {
          if (joyButtonPressedStates[VK_UP])
            joyButtonClickedStates[VK_UP] = true;
          if (joyButtonPressedStates[VK_DOWN])
            joyButtonClickedStates[VK_DOWN] = true;
          joyButtonPressedStates[VK_UP] = joyButtonPressedStates[VK_DOWN] = false;
        }
      }
      else
      {
        if (value < -32000) // up pressed
        {
          joyButtonPressedStates[VK_LEFT] = true;
          joyButtonClickedStates[VK_LEFT] = false;
        }
        else if (value > 32000) // down pressed
        {
          joyButtonPressedStates[VK_RIGHT] = true;
          joyButtonClickedStates[VK_RIGHT] = false;
        }
        else
        {
          if (joyButtonPressedStates[VK_LEFT])
            joyButtonClickedStates[VK_LEFT] = true;
          if (joyButtonPressedStates[VK_RIGHT])
            joyButtonClickedStates[VK_RIGHT] = true;
          joyButtonPressedStates[VK_LEFT] = joyButtonPressedStates[VK_RIGHT] = false;
        }
      }
      
      return true;
    }
    
    return false;
  }

  public boolean isButtonPressed(int button)
  {
    return joyButtonPressedStates[button];
  }
  
  public boolean isButtonClicked(int button)
  {
    boolean c = joyButtonClickedStates[button];
    joyButtonClickedStates[button] = false;
    return c;
  }
  
  public void resetButtons()
  {
    for (int i=0; i<joyButtonClickedStates.length; i++)
    {
      joyButtonClickedStates[i] = false;
      joyButtonPressedStates[i] = false;
    }
  }
  
  public SDLSurface getFramebuffer()
  {
    // return framebuffer;
    return backbuffer;
  }

  public SDLJoystick getJoystick()
  {
    return joystick;
  }

  public void repaint()
  {
    try
    {
      backbuffer.blitSurface(framebuffer);
      framebuffer.flip();
    }
    catch (SDLException e)
    {
      e.printStackTrace();
    }
  }
  
  public void clearScreen()
  {
    try
    {
      this.getFramebuffer().fillRect(0);
    }
    catch (SDLException e)
    {
      e.printStackTrace();
    }
  }
  
  public void drawImage(SDLSurface img, int x, int y)
  {
    try
    {
      img.blitSurface(this.getFramebuffer(), new SDLRect(x, y));
    }
    catch (SDLException e)
    {
      e.printStackTrace();
    }
  }

}
