/*
    $Id: StyleSheet.java,v 1.1.1.1 2005/11/14 17:33:29 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast;

import java.util.Vector;

/**
 * Provides a simple mechanism for setting user interface styles
 * that can be applied to objects that implement the Styled interface.
 * <P>
 * The toolkit has a default stylesheet that Styled classes may
 * register their default settings with (typically in a static
 * initialization block).
 * <P>
 * For simplicity and minimal size, all styles are defined in terms of
 * integer values.  A class implementing Styled is free to choose
 * arbitrary constants for both properties and values; a suggested
 * convention is the use of "public static final int" constant
 * declarations.  In addition, this class provides a pair of utility
 * methods for converting MIDP Font objects to and from integers.
 * <P>
 * Sample usage: 
 * <pre>
 * StyleSheet sheet = new StyleSheet();
 * sheet.setStyle(ColoredWidget.class, // or ._class for MIDP 1.0
 *   ColoredWidget.BORDER_COLOR, Color.BLUE)
 *      .setStyle(InputWidget.class, InputWidget.BORDER_COLOR,
 *   Color.GREEN); 
 * SynclastCanvas myCanvas = new SynclastCanvas(sheet);
 * </pre> */
public class StyleSheet {
    private static StyleSheet DEFAULT;

    /** Returns the current default stylesheet. */
    public static synchronized StyleSheet getDefault() {
        if (DEFAULT == null) {
            DEFAULT = new StyleSheet("Default");
        }
        return DEFAULT;
    }

    /** Sets the current default stylesheet. */
    public static void setDefault(StyleSheet newDefault) {
        DEFAULT = newDefault;
    }

    // Parallel vectors (order is important)
    private Vector classes = new Vector();
    // Vector of Vectors
    private Vector styleSets = new Vector();

    /**
     * Creates a new style sheet based on the default style sheet.
     */ 
    public StyleSheet() {
        this(getDefault());
    }

    /**
     * Creates a new style sheet by looking up settings from the MIDlet
     * environment.
     */
    public StyleSheet(String name) {
        String styles = SynclastManager.getProperty("Style-" + name);
        int p1, p2, p3, pos = 0;
        
        if (styles != null) {

            // This is the most inelegant code I've written in a long
            // time.  Please help me improve it.
            // The format is "ClassName1:propNo1=value1,propNo2=value2;ClassName2:propNo1=value1[,...;...]
            do {
                p1 = styles.indexOf(';', pos);
                p2 = styles.indexOf(':', pos);
                Class clazz = null;
                try {
                    clazz = Class.forName(styles.substring(pos, p2));
                } catch (Exception e) {
                    e.printStackTrace();
                }
                p3 = p2 + 1;
                pos = p1;
                if (pos == -1) { pos = styles.length(); }
                do {
                    p3 = styles.indexOf(',', p2 + 1);
                    String nameVal;
                    if ((p3 != -1) && (p3 < pos)) {
                        nameVal = styles.substring(p2 + 1, p3);
                    } else {
                        if (p1 != -1) {
                            nameVal = styles.substring(p2 + 1, p1);
                        } else {
                            nameVal = styles.substring(p2 + 1);
                        }
                    }
                    
                    // Now split nameVal...
                    int p4 = nameVal.indexOf('=');
                    // Set the indicated style
                    setStyle(clazz, 
                             Integer.parseInt(nameVal.substring(0,p4)),
                             Integer.parseInt(nameVal.substring(p4+1)));

                    p2 = p3;
                } while ((p3 != -1) && (p3 < pos));
                pos = p1 + 1;
            } while (p1 != -1);
        }
    }

    /**
     * Creates a new style sheet with the given source style sheet.
     */
    public StyleSheet(StyleSheet source) {
        if (source != null) {
            update(source);
        }
    }

    /**
     * Copies all the values from the other stylesheet into this one.
     */
    public void update(StyleSheet other) {
        int len = other.classes.size();
        for (int i = 0; i < len; i++) {
            Class type = (Class) other.classes.elementAt(i);
            Vector styles = (Vector) other.styleSets.elementAt(i);
            int jlen = styles.size();
            for (int j = 0; j < jlen; j++) {
                int[] propVal = (int[]) styles.elementAt(j);
                setStyle(type, propVal[0], propVal[1]);
            }
        }
    }

    /** 
     * Sets a single style property to the designated value.  The
     * value will be applied to instances of the specified Java class
     * and its subclasses only.
     */
    public synchronized StyleSheet setStyle(Class type, int property, int value) {
        int pos = classes.indexOf(type);
        Vector styles;
        if (pos == -1) {
            // Need to add a new entry to both vectors
            int len = classes.size();
            Class clazz;
            while (++pos < len) {
                // Insert it BEFORE anything that is a subclass of it.
                clazz = (Class) classes.elementAt(pos);
                if (type.isAssignableFrom(clazz)) {
                    break;
                }
            }
            classes.insertElementAt(type, pos);
            styles = new Vector();
            styleSets.insertElementAt(styles, pos);
        } else {
            styles = (Vector) styleSets.elementAt(pos);
        }
        // There can be only one entry for a given Class/property combo
        int vlen = styles.size();
        int j = 0;
        for (; j < vlen; j++) {
            int[] style = (int[]) styles.elementAt(j);
            if (style[0] == property) {
                style[1] = value;
                return this;
            }
        }
        styles.addElement(new int[] { property, value });
        return this;
    }

    /**
     * Retrieves the style value for the designated class and property.
     * If no value has been registered, the constant -1 is returned.
     */
    public int getStyle(Class type, int property) {
        int pos = classes.indexOf(type);
        if (pos == -1) {
            Vector v = (Vector) styleSets.elementAt(pos);
            int len = v.size();
            int[] style;
            for (int i = 0; i < len; i++) {
                style = (int[]) v.elementAt(i);
                if (style[0] == property) {
                    return style[1];
                }
            }
        }
        return -1;
    }

    /**
     * Applies this stylesheet to the argument object.
     * This will trigger 0 or more calls to Styled.setDefault(property, value)
     * depending on the properties that have been registered with this
     * stylesheet for the particular Styled class.
     */
    public void applyTo(Styled styled) {
        int len = classes.size();
        for (int i = 0; i < len; i++) {
            Class clazz = (Class) classes.elementAt(i);
            if (clazz.isAssignableFrom(styled.getClass())) {
                Vector v = (Vector) styleSets.elementAt(i);
                int vlen = v.size();
                for (int j = 0; j < vlen; j++) {
                    int[] style = (int[]) v.elementAt(j);
                    styled.setDefault(style[0], style[1]);
                }
            }
        }
    }

    /**
     * Returns a single int representing a MIDP Font, for
     * use with font style properties.
     */
    public static int getFontValue(Font font) {
        return font.getStyle() | font.getSize() | font.getFace();
    }

    /**
     * Returns the font represented by an integer value, the
     * inverse operation of getFontValue().
     */
    public static Font getFont(int value) {
        return Font.getFont(value & 96, value & 7, value & 24);
    }
}
