/*
    $Id: ColoredWidget.java,v 1.3 2005/11/24 17:32:25 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui;

import com.synclast.Graphics;
import com.synclast.Image;

/**
 * Base class for widgets that are rectangular or rounded and have
 * customizable colors.  A ColoredWidget specifies a foreground color,
 * background color, and borderColor.  Foreground defaults to black;
 * background and border are off by default (transparent).  If a
 * border is indicated, it is drawn using the borderArc and
 * borderWidth properties.
 *
 * If a background image is indicated, it is drawn either tiled
 * (the default) or centered.  The image's upper left hand corner
 * will be positioned inside the border, if present.
 */
public class ColoredWidget extends Widget {
    /** Constant for style sheet property. */
    public static final int FOREGROUND_COLOR = Widget.LAST_PROPERTY + 1;
    /** Constant for style sheet property. */
    public static final int BACKGROUND_COLOR = Widget.LAST_PROPERTY + 2;
    /** Constant for style sheet property. */
    public static final int BORDER_COLOR = Widget.LAST_PROPERTY + 3;
    /** Constant for style sheet property. */
    public static final int BORDER_ARC = Widget.LAST_PROPERTY + 4;
    /** Constant for style sheet property. */
    public static final int BORDER_WIDTH = Widget.LAST_PROPERTY + 5;
    /** Constant for style sheet property. */
    public static final int IMAGE_MODE = Widget.LAST_PROPERTY + 6;
    /** Constant for style sheet property. */
    public static final int INSET_WIDTH = Widget.LAST_PROPERTY + 7;

    protected static final int LAST_PROPERTY = INSET_WIDTH;

    // Workaround for MIDP 1.0 .class access
    public static Class _class = new ColoredWidget().getClass();

    /** Constant for IMAGE_MODE property. */
    public static final int TILED = 0;
    /** Constant for IMAGE_MODE property. */
    public static final int CENTERED = 1;

    protected Image backgroundImage;

    public ColoredWidget() { }

    public ColoredWidget(int width, int height) {
        this.w = width;
        this.h = height;
    }

    protected int getPropertyCount() {
        return LAST_PROPERTY + 1;
    }

    protected void init() {
        super.init();
        set(BACKGROUND_COLOR, Color.TRANSPARENT);
        set(BORDER_COLOR, Color.TRANSPARENT);
    }

    /**
     * If the background color is not transparent, paint the
     * background area.  If the border is not transparent, paint
     * the border.  When this call returns, the graphics instance's
     * color property will be set to the desired foreground color.
     * If a visible border is specified, the graphics object will be
     * further clipped.
     */
    public void paint(Graphics graphics) {
      if (this.invisible)
        return;
      
        int borderWidth = properties[BORDER_WIDTH];
        if (backgroundImage == null || properties[IMAGE_MODE] == CENTERED) {
            if (properties[BACKGROUND_COLOR] != Color.TRANSPARENT) {
                graphics.setColor(properties[BACKGROUND_COLOR]);
                graphics.fillRoundRect(borderWidth, borderWidth,
                                       w - borderWidth,
                                       h - borderWidth, 
                                       properties[BORDER_ARC],
                                       properties[BORDER_ARC]);
            }
        }
        if (backgroundImage != null) {
            switch (properties[IMAGE_MODE]) {
            case CENTERED:
                graphics.drawImage(backgroundImage, w/2, h/2,
                                   Graphics.HCENTER | Graphics.VCENTER);
                break;
            case TILED:
                int bgWidth = backgroundImage.getWidth();
                int bgHeight = backgroundImage.getHeight();
                if (bgWidth > 0 && bgHeight > 0) {
                    int cols = ((w - 1) / bgWidth) + 1;
                    int rows = ((h - 1) / bgHeight) + 1;
                    while (rows-- > 0) {
                        for (int i = 0; i < cols; i++) {
                            graphics.drawImage
                                (backgroundImage, 
                                 borderWidth + bgWidth * i,
                                 borderWidth + bgHeight * rows,
                                 Graphics.TOP | Graphics.LEFT);
                        }
                    }
                }
                break;
            }
        }
        if (borderWidth > 0) {
            if (properties[BORDER_COLOR] != Color.TRANSPARENT) {
                graphics.setColor(properties[BORDER_COLOR]);
                for (int i = 0; i < borderWidth; i++) {
                    graphics.drawRoundRect(i,i,w-1-i*2,h-1-i*2, 
                                           properties[BORDER_ARC],
                                           properties[BORDER_ARC]);
                }
            }
        }
        
        int totalInset = getTotalInset();
        if (totalInset > 0) {
            graphics.clipRect(totalInset, totalInset,
                              w - 2 * totalInset,
                              h - 2 * totalInset);
            graphics.translate(totalInset, totalInset);
        }
        graphics.setColor(properties[FOREGROUND_COLOR]);
    }

    public Image getBackgroundImage() {
        return backgroundImage;
    }

    public void setBackgroundImage(Image backgroundImage) {
        this.backgroundImage = backgroundImage;
        repaint();
    }

    protected int getTotalInset() {
        return properties[BORDER_WIDTH] + properties[INSET_WIDTH];
    }

    public Widget set(int property, int value) {
        if ((property == BORDER_WIDTH) || (property == INSET_WIDTH)) {
            int delta = 2 * (value - properties[property]);
            w += delta;
            h += delta;
        }
        return super.set(property, value);
    }
}
