/*
    $Id: Input.java,v 1.2 2005/11/24 17:32:25 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui;

import java.util.Vector;


import com.synclast.Font;
import com.synclast.Graphics;
import com.synclast.StyleSheet;
import com.synclast.SynclastManager;
import com.synclast.SynclastTask;

/**
 * Represents a basic text input widget.  An Input widget uses one or more
 * InputAdapter instances in order to translate user input into
 * characters.  If no adapter is otherwise specified, an instance
 * of the base class InputAdapter is used; this provides a straight
 * mapping of keycodes to characters.  
 * 
 * Applications that want to do text entry with a numeric keypad should
 * use the TapInputAdapter; two named instances are available by default,
 * and others can be configured via JAD properties.  See the TapInputAdapter
 * class for details.
 */
public class Input extends ColoredWidget 
    implements KeyEventListener, InputHandler, Runnable {
    // Workaround for MIDP 1.0 .class access
    public static Class _class = new Input().getClass();

    /**
     * Constant for style sheet property.  Setting to a non-zero value 
     * indicates the character code to be displayed for each character
     * in the text.
     */
    public static final int MASK_CHAR = ColoredWidget.LAST_PROPERTY + 1;

    public static final int FONT = ColoredWidget.LAST_PROPERTY + 2;
    public static final int CURSOR_BLINK_DELAY = ColoredWidget.LAST_PROPERTY + 3;
    public static final int MAX_SIZE = ColoredWidget.LAST_PROPERTY + 4;
    public static final int WIDTH = ColoredWidget.LAST_PROPERTY + 5;

    protected static final int LAST_PROPERTY = WIDTH;

    protected SynclastTask cursor;

    // Current cursor position, 0 = before first character
    protected int pos;

    // Horizontal pixel offset corresponding to current cursor position
    protected int pixelOffset;

    // Is the cursor currently visible?
    protected boolean on;

    protected StringBuffer text;
    protected Font font;

    protected Vector adapters;
    protected InputAdapter adapter;
    protected int adapterIndex;

    public Input() {
        this(SynclastManager.getProperty("Input-Default"));
    } 

    public Input(String adapterList) {
        this.text = new StringBuffer();
        if (adapterList != null) {
            setAdapters(adapterList);
        } else {
            adapter = new InputAdapter();
        }
    }

    protected void init() {
        super.init();
        set(BORDER_WIDTH, 1)
            .set(BORDER_COLOR, Color.BLACK)
            .set(INSET_WIDTH, 2)
            .set(CURSOR_BLINK_DELAY, 500)
            .set(MAX_SIZE, 10)
            .set(FONT, StyleSheet.getFontValue(Font.getDefaultFont()));
    }

    public void setAdapters(String adapterList) {
        // Divide on ','
        int pos = 0;
        int pos2;
        int len = adapterList.length();
        adapters = new Vector();
        while (pos < len) {
            pos2 = adapterList.indexOf(',', pos);
            if (pos2 != -1) {
                adapters.addElement(InputAdapter.getNamedInstance(adapterList.substring(pos, pos2).trim()));
                pos = pos2 + 1;
            } else {
                adapters.addElement(InputAdapter.getNamedInstance(adapterList.substring(pos).trim()));
                break;
            }
        }
        adapterIndex = 0;
        adapter = (InputAdapter) adapters.elementAt(0);
        if (on) {
            adapter.setHandler(this);
        }
    }

    /**
     * Returns the current user-entered string value.
     */
    public String getString() {
        return text.toString();
    }

    public int getMaxSize() {
        return properties[MAX_SIZE];
    }

    protected int getPropertyCount() {
        return LAST_PROPERTY + 1;
    }

    public void paintNoText(Graphics graphics) {
      super.paint(graphics);
    }
    
    public void paint(Graphics graphics) {
      paintNoText(graphics);
      
        // Draw the text
        if (text.length() > 0) {
            char[] temp = text.toString().toCharArray();
            if (properties[MASK_CHAR] != 0) {
                for (int i = 0; i < temp.length; i++) {
                    temp[i] = (char) properties[MASK_CHAR];
                }
            }
	    
            graphics.setFont(font);
            graphics.drawChars(temp, 0, temp.length, 0, 0, 
                               Graphics.TOP | Graphics.LEFT);
        }

        // Draw the cursor, if it's on
        if (on) {
            graphics.fillRect(pixelOffset, 1, 
                              2, font.getBaselinePosition() + 1);
        }
    }

    public void backspace() {
        if (pos != 0) {
            pixelOffset -= font.charWidth(properties[MASK_CHAR] != 0 ? 
                                          (char) properties[MASK_CHAR] 
                                          : text.charAt(pos - 1));
            text.deleteCharAt(--pos);
            repaint();
        }
    }

    public void addCharacter(char ch) {
        if (pos < properties[MAX_SIZE]) {
            text.insert(pos++, ch);
            if (properties[MASK_CHAR] != 0) {
                ch = (char) properties[MASK_CHAR];
            }
            pixelOffset += font.charWidth(ch);
            repaint();
        }
    }

    public void setString(String s) {
        text  = new StringBuffer(s);
        pos = pixelOffset = 0;
        moveCursor(text.length());
    }

    public UIContext getContext() {
        return container.getContext();
    }

    public boolean changeMode() {
        if (adapters == null || adapters.size() == 1) {
            return false;
        }

        if (++adapterIndex == adapters.size()) {
            adapterIndex = 0;
        }
        adapter = (InputAdapter) adapters.elementAt(adapterIndex);
        adapter.setHandler(this);

        // Flash the new mode name
        // TODO make this more configurable
        Label modeInfo = new Label(adapter.title, Font.getDefaultFont());
        modeInfo.set(BACKGROUND_COLOR, Color.WHITE);
        Popup popup = new Popup(modeInfo, 1000);
        popup.setPosition(x + w/2, y + h/2,
                          Graphics.HCENTER | Graphics.VCENTER);
        container.add(popup);
        return true;
    }

    public void passFocus(int direction) {
        container.passFocus(direction);
    }

    public void moveCursor(int direction) {
        while (direction < 0) {
            if (pos > 0) {
                --pos;
                pixelOffset -= font.charWidth(properties[MASK_CHAR] != 0 ? 
                                              (char) properties[MASK_CHAR] 
                                              : text.charAt(pos));
            }
            direction++;
        }
        while (direction > 0) {
            if (pos < text.length()) {
                pixelOffset += font.charWidth(properties[MASK_CHAR] != 0 ? 
                                              (char) properties[MASK_CHAR] 
                                              : text.charAt(pos));
                pos++;
            }
            direction--;
        }
    }
    
    public void keyPressed(int keyCode) {
        adapter.keyPressed(keyCode);
    }

    public void keyReleased(int keyCode) {
        adapter.keyReleased(keyCode);
    }

    public void keyRepeated(int keyCode) {
        adapter.keyPressed(keyCode);
    }

    public void focus() {
        on = true;
        properties[BORDER_WIDTH]++;
        properties[INSET_WIDTH]--;
        repaint();
        int delay = properties[CURSOR_BLINK_DELAY];
        if (delay > 0) {
            cursor = new SynclastTask(this, delay, delay, true);
        } else if (delay < 0) {
            on = false;
        }
        adapter.setHandler(this);
    }

    public void blur() {
        properties[BORDER_WIDTH]--;
        properties[INSET_WIDTH]++;
        if (cursor != null) {
            cursor.cancel();
            cursor = null;
        }
        on = false;
        repaint();
    }

    public Widget set(int property, int value) {
        if (property == FONT) {
            if (font != null) {
                if (properties[WIDTH] == 0) {
                    w -= properties[MAX_SIZE] * font.charWidth('x');
                }
                h -= font.getHeight();
            }
            font = StyleSheet.getFont(value);
            if (properties[WIDTH] == 0) {
                w += properties[MAX_SIZE] * font.charWidth('x');
            }
            h += font.getHeight();
            // Recalculate pixelOffset
            if (text != null) {
                if (properties[MASK_CHAR] == 0) {
                    pixelOffset = font.substringWidth(text.toString(), 0, pos);
                } else {
                    pixelOffset = pos * font.charWidth((char) properties[MASK_CHAR]);
                }
            }
            invalidate();
        } else if (property == WIDTH) {
            w = value;
            invalidate();
        } else if (property == MAX_SIZE) {
            if ((font != null) && (properties[WIDTH] == 0)) {
                w -= properties[MAX_SIZE] * font.charWidth('x');
                w += value * font.charWidth('x');
                invalidate();
            }
        }
        return super.set(property, value);
    }

    // Blink cursor while activated
    public void run() {
        on = !on;
        repaint();
    }
}
