/*
    $Id: InputAdapter.java,v 1.3 2005/11/24 17:32:25 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui;

import java.util.Hashtable;
import com.synclast.BaseCanvas;
import com.synclast.SynclastManager;

/**
 * Base class for implementations of various input methods.
 * An InputAdapter receives keypress events and translates them into
 * characters and control events, typically in coordination with a widget
 * that implements the InputHandler interface and is responsible for
 * maintaining the graphical view of the input item.
 * 
 * This implementation is pass-through and direct-mode; all printable
 * characters will be entered corresponding to their keycodes' character
 * equivalents.  For numeric keypads, this means 0-9, * and # are available;
 * for QWERTY keyboards, the full character set may be available.  For
 * text input via a numeric (e.g. phone) keypad, see the TapInputAdapter
 * class.
 */
public class InputAdapter {
    private static Hashtable NAMED_ADAPTERS;
    private static Hashtable DEFAULTS;

    static { 
        NAMED_ADAPTERS = new Hashtable();
        DEFAULTS = new Hashtable();
        DEFAULTS.put("Numeric", "123,com.synclast.ui.InputAdapter,");
        DEFAULTS.put("Tap-Upper", "ABC,com.synclast.ui.TapInputAdapter,/@0,.\\,?!1'-_():;&%*#+<=>\\\"$,ABC2,DEF3,GHI4,JKL5,MNO6,PQRS7,TUV8,WXYZ9");
        DEFAULTS.put("Tap-Lower", "abc,com.synclast.ui.TapInputAdapter,/@0,.\\,?!1'-_():;&%*#+<=>\\\"$,abc2,def3,ghi4,jkl5,mno6,pqrs7,tuv8,wxyz9");
    }

    public synchronized static void registerInstance(String name, InputAdapter adapter) {
        NAMED_ADAPTERS.put(name, adapter);
    }

    public synchronized static InputAdapter getNamedInstance(String name) {
        InputAdapter adapter = (InputAdapter) NAMED_ADAPTERS.get(name);
        if (adapter == null) {
            // Property formatted as title,com.xyz.class,configurationArgs
            String value = SynclastManager.getProperty("Input-" + name);
            if (value == null) {
                value = (String) DEFAULTS.get(name);
            }
            if (value != null) {
                int i1 = value.indexOf(',');
                String title = value.substring(0, i1++);
                int i2 = value.indexOf(',', i1);
                String className = value.substring(i1, i2++);
                
                try {
                    adapter = (InputAdapter) Class.forName(className)
                        .newInstance();
                    adapter.title = title;
                    adapter.configure(value.substring(i2));
                } catch (Exception e) {
                    // Ignore, return null?
                    e.printStackTrace();
                }
            }
            if (adapter != null) {
                NAMED_ADAPTERS.put(name, adapter);
            }
        }
        return adapter;
    }

    protected int CLEAR = -8;
    protected int SHIFT = (int) '*';

    protected InputHandler handler;
    protected String title;

    /**
     * The default implementation requires no parameters, but can take
     * {clearKeyCode},{shiftKeyCode}.  These default to -8 and 42 (the
     * values used in the Wireless ToolKit and many phones).  Note that
     * if there are no other modes specified, the shift key ('*') will
     * come through as a literal.
     */
    public void configure(String parameters) { 
        int pos = parameters.indexOf(',');
        if (pos != -1) {
            CLEAR = Integer.parseInt(parameters.substring(0, pos));
            SHIFT = Integer.parseInt(parameters.substring(pos + 1));
        }
    }

    /** 
     * This can be considered a focus() event.  It is called whenever
     * the adapter is put into use for a widget.
     */
    public void setHandler(InputHandler handler) {
        this.handler = handler;
    }

    public void keyPressed(int keyCode) {

      /*
        if ((keyCode == SHIFT) && handler.changeMode()) {
            return;
        }
      */
        if (keyCode == CLEAR) {
            handler.backspace();
        } else if ((keyCode >= ' ') && (keyCode <= '~')) {
            handler.addCharacter((char) keyCode);
        } else {
            switch (handler.getContext().getGameAction(keyCode)) {
            case BaseCanvas.UP:
                handler.passFocus(-1);
                return;
            case BaseCanvas.DOWN:
                handler.passFocus(1);
                return;
            case BaseCanvas.LEFT:
                handler.moveCursor(-1);
                return;
            case BaseCanvas.RIGHT:
                handler.moveCursor(1);
                return;
            case BaseCanvas.FIRE:
                handler.changeMode();
                return;
            }
            // System.out.println("Unrecognized keyCode: " + keyCode);
        }
    }

    /** The default implementation does nothing with this event. */
    public void keyReleased(int keyCode) { }

    /** The default implementation treats a repeat the same as keyPressed(). */
    public void keyRepeated(int keyCode) { 
        keyPressed(keyCode);
    }
    public String getTitle()
    {
      return title;
    }
    public void setTitle(String title)
    {
      this.title = title;
    }
    public InputHandler getHandler()
    {
      return handler;
    }
}
