/*
    $Id: Menu.java,v 1.5 2005/11/24 17:32:25 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui;

import java.util.Vector;


import com.synclast.Command;
import com.synclast.Event;
import com.synclast.BaseCanvas;
import com.synclast.Font;
import com.synclast.Graphics;

/**
 * Represents a menu of selectable items.  The items may map to
 * specific (fixed) keys.  The currently selected item is displayed
 * in bold font, using the selectedColor attribute (defaults to black).
 * Other items are displayed using defaultColor (also defaults to black),
 * without any bolding.  The default background color is transparent.
 * The horizontal alignment of the menu can be set if required; it
 * defaults to HCENTER.
 *
 * Both horizontal and vertical arrow keys will cause the selected item
 * to change.  The selected item is allowed to wrap around between the
 * top and the bottom of the list.
 *
 * This element autosizes to match the width and height of its contents.
 * This value may be overridden if necessary.
 *
 * When an item is selected, either by the FIRE game key or by a
 * mapped keyCode, the selected index is set and the List.SELECT_COMMAND
 * command is wrapped in an Event object and sent to the CommandListener's
 * commandAction() method.  A typical command listener implementation
 * will look something like this:
 *
 * <pre>
 * public void commandAction(Command command, Displayable displayable) {
 *   if (displayable == mySynclastCanvas) {
 *     if (Event.getSource(command) == myMenu) {
 *       int selected = myMenu.getSelectedIndex();
 *       // do something useful
 *     }
 *   }
 * }
 * </pre>
 */
public class Menu extends ColoredWidget implements KeyEventListener {
  protected static final Font FONT_BOLD = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_BOLD, Font.SIZE_MEDIUM);
  protected static final Font FONT_DEFAULT = Font.getDefaultFont();

  public static final int SELECTED_FOREGROUND_COLOR = ColoredWidget.LAST_PROPERTY + 1;
  public static final int SELECTED_BACKGROUND_COLOR = ColoredWidget.LAST_PROPERTY + 2;
  public static final int FOCUSED_BORDER_COLOR = ColoredWidget.LAST_PROPERTY + 3;
  public static final int UNFOCUSED_BORDER_COLOR = ColoredWidget.LAST_PROPERTY + 4;
    public static final int ITEM_ALIGN = ColoredWidget.LAST_PROPERTY + 5;

    protected static final int LAST_PROPERTY = ITEM_ALIGN;

    // Workaround for MIDP 1.0 .class access
    public static Class _class = new Menu().getClass();

    protected int getPropertyCount() {
        return LAST_PROPERTY + 1;
    }

    protected Vector items = new Vector();
    protected Vector keys = new Vector();
    protected int selected;

    protected void init() {
        super.init();
        set(INSET_WIDTH, 2);
        set(ITEM_ALIGN, Graphics.HCENTER);
        set(SELECTED_FOREGROUND_COLOR, Color.BLACK);
        set(SELECTED_BACKGROUND_COLOR, Color.WHITE);
        set(FOCUSED_BORDER_COLOR, Color.BLACK);
        set(UNFOCUSED_BORDER_COLOR, Color.WHITE);
    }

    public int getSelectedIndex() {
        return selected;
    }

    public String getString(int index) {
        return (String) items.elementAt(index);
    }

    public int getKeyCode(int index) {
        return ((Integer) items.elementAt(index)).intValue();
    }

    public void paint(Graphics graphics) {
        super.paint(graphics);
        for (int i = 0; i < items.size(); i++) {
            graphics.setColor(i == selected ? properties[SELECTED_FOREGROUND_COLOR] 
                              : properties[FOREGROUND_COLOR]);
            graphics.setFont(i == selected ? FONT_BOLD : FONT_DEFAULT);
            int hpos = 0;
            if (properties[ITEM_ALIGN] == Graphics.HCENTER) {
                hpos = (w - 2 * getTotalInset()) / 2;
            } else if (properties[ITEM_ALIGN] == Graphics.RIGHT) {
                hpos = w - 2 * getTotalInset();
            }
            graphics.drawString((String) items.elementAt(i), hpos,
                                FONT_DEFAULT.getHeight() * i, 
                                properties[ITEM_ALIGN] | Graphics.TOP);
        }
    }

    public void add(String item) {
        add(item, 0);
    }

    public void add(String item, int keyCode) {
        items.addElement(item);
        keys.addElement(new Integer(keyCode));
        h += FONT_DEFAULT.getHeight();
        int cw = FONT_BOLD.charsWidth(item.toCharArray(), 0, item.length())
            + 2 * getTotalInset();
        if (cw > w) { w = cw; }
        invalidate();
    }

    public void keyPressed(int keyCode) {
        // See if it's a user-registered keyCode
        for (int i = 0; i < keys.size(); i++) {
            if (((Integer) keys.elementAt(i)).intValue() == keyCode) {
                selected = i;
                fireEvent(new Event(this, Command.SELECT_COMMAND));
                repaint();
                return;
            }
        }
        switch (container.getContext().getGameAction(keyCode)) {
        case BaseCanvas.UP:
            if (selected == 0) selected = items.size();
            selected--;
            repaint();
            break;
        case BaseCanvas.LEFT:
            container.passFocus(-1);
            break;
        case BaseCanvas.DOWN:
            selected++;
            if (selected == items.size()) selected = 0;
            repaint();
            break;
        case BaseCanvas.RIGHT:
            container.passFocus(1);
            break;
        case BaseCanvas.FIRE:
            fireEvent(new Event(this, Command.SELECT_COMMAND));
        }
    }

    public void keyRepeated(int keyCode) { 
        keyPressed(keyCode);
    }

    public void keyReleased(int keyCode) { }
    public void focus() { }
    public void blur() { }
}
