/*
    $Id: SynclastCanvas.java,v 1.3 2005/11/18 17:54:12 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui;

import com.synclast.BaseCanvas;

import com.synclast.CommandListener;
import com.synclast.Event;
import com.synclast.Graphics;
import com.synclast.Image;
import com.synclast.StyleSheet;
import com.synclast.ui.containers.Container;

/**
 * This is the centerpiece of the Synclast user interface toolkit.
 * A SynclastCanvas contains Widgets to which it delegates rendering
 * and event handling tasks.
 */
public class SynclastCanvas extends BaseCanvas implements UIContext {
    protected Container container;
    protected StyleSheet styleSheet;
    protected int bgColor = Color.WHITE;
    protected CommandListener commandListener;

    // Used for double buffering if not natively supported
    protected Image imgBuffer;

    /**
     * Creates a new canvas using the default style sheet
     * and default (absolute positioning) container.
     */
    public SynclastCanvas() {
        this(new Container(), StyleSheet.getDefault());
    }

    /**
     * Creates a new canvas using the default style sheet
     * and the specified container.
     */
    public SynclastCanvas(Container container) {
        this(container, StyleSheet.getDefault());
    }

    /**
     * Creates a new canvas using the specified style sheet
     * and default container.
     */
    public SynclastCanvas(StyleSheet styleSheet) {
        this (new Container(), styleSheet);
    }

    public SynclastCanvas(Container container, StyleSheet styleSheet) {
        container.context = this;
        this.container = container;
        setStyleSheet(styleSheet);
        if (!isDoubleBuffered()) {
            imgBuffer = Image.createImage(getWidth(), getHeight());
        }
    }

    /** 
     * Replaces the top-level container for this canvas.
     */
    public void setContainer(Container container) {
        container.context = this;

        // Detach old container
        this.container.context = null;

        // Apply the old stylesheet to the new container
        this.container = container;
        container.applyStyleSheet(styleSheet);
    }

    /**
     * Gets a reference to the container in use
     * as the top-level container.
     */
    public Container getContainer() {
        return container;
    }

    /**
     * Applies the given style sheet to all widgets on the canvas.
     */
    public void setStyleSheet(StyleSheet styleSheet) {
        this.styleSheet = styleSheet;
        container.applyStyleSheet(styleSheet);
    }

    /** Returns a reference to the StyleSheet in use. */
    public StyleSheet getStyleSheet() {
        return styleSheet;
    }

    /**
     * Sets the background color of the canvas.  The default
     * background color is white.
     */
    public SynclastCanvas setBackgroundColor(int color) {
        bgColor = color;
        repaint();
        return this;
    }

    /** Returns the current background color of the canvas. */
    public int getBackgroundColor() {
        return bgColor;
    }

    /**
     * Paints all widgets in the graphics clip area,
     * beginning with the background color of the canvas itself,
     * if it is non-transparent.  The widgets will be painted in the
     * same order they were added to the canvas; therefore, widgets
     * added later may overlap or cover earlier widgets.
     * The paint() method is only called on a given widget if the
     * widget overlaps the clipping rectangle.
     */
    public void paint(Graphics g) {
        Graphics screenGraphics = g;
        if (imgBuffer != null) {
            g = imgBuffer.getGraphics();
        }
        // Paint the background color
        if (bgColor != Color.TRANSPARENT) {
            g.setColor(bgColor);
            g.fillRect(0, 0, getWidth(), getHeight());
        }
        // Translate for container if necessary
        int align = container.properties[Widget.ALIGN];

        // Default is top left
        int dx = 0;
        int dy = 0;
        if (align != 0) {
            // Make sure container width and height are correct
            if (container.invalid) {
                container.validate();
            }
            if ((align & Graphics.LEFT) == 0) {
                dx = getWidth() - container.w;
                if ((align & Graphics.HCENTER) != 0) {
                    dx /= 2;
                }
            }
            if ((align & Graphics.TOP) == 0) {
                dy = getHeight() - container.h;
                if ((align & Graphics.VCENTER) != 0) {
                    dy /= 2;
                }
            }
            g.translate(dx, dy);
        }
        container.paint(g);
        if (align != 0) {
            g.translate(-dx, -dy);
        }
        if (imgBuffer != null) {
            screenGraphics.drawImage(imgBuffer, 0, 0, Graphics.TOP | Graphics.LEFT);
        }
    }

    /**
     * Adds a new widget to the canvas, positioning it according
     * to the layout rules of the top-level container.
     * The first action taken is to apply the current style sheet
     * to the widget.  Next, the widget has its setContainer() method
     * called.
     *
     * @return this SynclastCanvas object for convenient chaining
     */
    public SynclastCanvas add(Widget widget) {
        container.add(widget);
        return this;
    } 

    /**
     * Removes the specified widget from the canvas.
     * This call forces a repaint of the canvas.
     *
     * @param widget the widget to remove.  If the widget is currently
     * not added to the canvas, no action is taken.
     */
    public void remove(Widget widget) {
        container.remove(widget);
    }

    /**
     * Dispatches a keyReleased event.  If there is a currently
     * focused widget, that widget is given a chance to handle the event.
     */
    public void keyReleased(int keyCode) {
        container.keyReleased(keyCode);
    }

    /**
     * Dispatches a keyRepeated event.  If there is a currently
     * focused widget, that widget is given a chance to handle the event.
     */
    public void keyRepeated(int keyCode) {
        container.keyRepeated(keyCode);
    }

    /**
     * Dispatches a keyPressed event.  If there is a currently
     * focused widget, that widget is given a chance to handle the event.
     */
    public void keyPressed(int keyCode) {
        container.keyPressed(keyCode);
    }

    /**
     * Dispatches a pointerPressed event.
     * All widgets that implement the PointerEventListener interface
     * have their pointerPressed method called until one returns true.
     */
    public void pointerPressed(int x, int y) {
        container.pointerPressed(x, y);
    }

    /**
     * Dispatches a pointerDragged event.
     * All widgets that implement the PointerEventListener interface
     * have their pointerDragged method called until one returns true.
     */
    public void pointerDragged(int x, int y) {
        container.pointerDragged(x, y);
    }

    /**
     * Dispatches a pointerReleased event.
     * All widgets that implement the PointerEventListener interface
     * have their pointerReleased method called until one returns true.
     */
    public void pointerReleased(int x, int y) {
        container.pointerReleased(x, y);
    }

    public void showNotify() {
        container.showNotify();
        container.focus();
    }

    public void hideNotify() {
        container.blur();
        container.hideNotify();
    }

    public void setCommandListener(CommandListener commandListener) {
        this.commandListener = commandListener;
    }

    public boolean fireEvent(Event event) {
        if (commandListener != null) {
            commandListener.commandAction(event, this);
            return true;
        }
        return false;
    }
}
