/*
    $Id: Widget.java,v 1.2 2005/11/16 11:51:55 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui;


import com.synclast.CommandListener;
import com.synclast.Event;
import com.synclast.Graphics;
import com.synclast.StyleSheet;
import com.synclast.Styled;
import com.synclast.ui.containers.Container;

/**
 * The base class for all widgets.  The widget lifecycle is as follows:
 * First, the instance is constructed by the user.  At some point, it is
 * added to a Container (a widget can only be associated with a single
 * container at any point in time).  This causes the container's style
 * sheet to be applied to the widget, which may involve any number of
 * invocations of set(property, value).  Next, the setContainer()
 * method is invoked.  After the setContainer() invocation, the
 * widget's internal state for width and height should
 * be set to the amount of space the widget is requesting.  If a
 * widget is removed from its container, setContainer(null) is called.
 * <p> 
 * A Widget can be thought of as a close cousin to the Displayable
 * object and can therefore have CommandListeners.  Widget subclasses
 * can fire events based on specific user interactions by invoking
 * commandAction() on their commandListener.  This is typically done
 * by using the Event class to wrap the command so the source widget
 * can be determined.  The fireEvent() method provides a convenient
 * implementation of this approach that can be reused by subclasses. 
 */
public abstract class Widget implements Styled {
    /** Constant for style sheet property. */
    public static final int ALIGN = 0;

    protected static final int LAST_PROPERTY = ALIGN;

    // X,Y coordinates are in container's coordinate system
    // width and height are in pixels
    public int x, y, w, h;

    public int[] properties;
    public boolean invalid;

    long setBits;
    protected Container container;
    protected CommandListener commandListener;
    protected boolean invisible;

    /**
     * Performs base-level widget initialization.  No
     * settings are applied, but the getPropertyCount()
     * method is applied to determine the size of the
     * properties array, which is initialized.
     */
    protected Widget() {
        init();
        setBits = 0L;
    }

    protected int getPropertyCount() {
        return LAST_PROPERTY + 1;
    }

    /**
     * Gives the widget an opportunity to initialize to default values. 
     * This method is called when the widget is first constructed,
     * and later if the current StyleSheet is removed.
     * Widget subclasses should override init() if they need to do
     * any default initialization of properties.  The implementation
     * of this method on the Widget class itself initializes the
     * properties array.  Subclasses should
     * always call super.init() at the beginning of their init() method.
     */
    protected void init() {
        properties = new int [getPropertyCount()];
    }

    /**
     * Returns true if this widget's paint() method should be called
     * during a repaint() cycle.
     */
    public boolean isVisible() {
        return !invisible;
    }

    /**
     * Sets the visibility status of this widget.  Changes to visibility
     * cause this widget's area of the container to be repainted.
     */
    public void setVisible(boolean visible) {
        boolean changed = (invisible == visible);
        if (changed && (container != null)) {
            invisible = !visible;
            invalidate();
        }
    }

    /** Returns the current width of this widget. */
    public int getWidth() {
        return w;
    }

    /** Returns the current height of this widget. */
    public int getHeight() {
        return h;
    }

    /** Returns the current X coordinate of this widget. */
    public int getX() {
        return x;
    }

    /** Returns the current Y coordinate  of this widget. */
    public int getY() {
        return y;
    }

    /** Returns the current Y coordinate  of this widget. */
    public int getBaseline() {
        return h+y;
    }

    /**
     * Sets the command listener associated with this widget.
     * If no command listener is set, and this widget generates
     * a command or event, it will pass the command to its parent
     * to be fired.
     */
    public void setCommandListener(CommandListener commandListener) {
        this.commandListener = commandListener;
    }

    /**
     * Fires a command wrapped in an Event object.  The
     * commandAction() method of the listener (if present) is
     * invoked with the SynclastCanvas as the Displayable.
     *
     * @return true if a command listener was found
     */
    protected boolean fireEvent(Event event) {
        if (commandListener != null) {
            if (container != null && container.getContext() != null) {
                return container.getContext().fireEvent(event);
            }
        } else if (container != null) {
            return container.fireEvent(event);
        }
        return false;
    }

    /**
     * Paints the widget onto its specified area.  The graphics
     * object passed to this method will be set to clip to the
     * exact area of the widget, and its coordinates will be
     * translated such that point (0,0) is the upper left corner
     * of the widget.  Subclasses must provide the implementation
     * of this method.
     */
    public abstract void paint(Graphics graphics);

    /**
     * Sets the container that is managing this widget.  A widget
     * can only be managed by a single container at a given time.
     * Subclasses may override this method to perform custom
     * initialization based on container attributes; they should
     * first call super.setContainer().
     */
    public void setContainer(Container container) {
        this.container = container;
    }

    /**
     * Applies the given StyleSheet to the widget.  A null
     * argument indicates that the widget should revert to
     * its default values.  Otherwise, sheet.applyTo(this)
     * can be invoked to populate this widget's properties
     * via the set() method.
     */
    public void applyStyleSheet(StyleSheet sheet) {
        if (sheet != null) {
            sheet.applyTo(this);
        } else {
            init();
            setBits = 0L;
        }
        invalidate();
    }

    /**
     * Sets a specified property if it has no user-provided
     * value (via the set(property, value) method).  If so,
     * this method passes its parameters to set(property, value)
     * but clears the explicit setting flag for the property
     * when complete.
     */
    public final void setDefault(int property, int value) {
        if ((setBits & (1 << property)) == 0) {
            set(property, value);
            setBits &= ~(1 << property);
        }
    }

    /**
     * Sets a specified property to the given value.  Subclasses
     * should call super.set(property, value) after any internal
     * processing.
     */
    public Widget set(int property, int value) {
        properties[property] = value;
        setBits |= (1 << property);
        invalidate();
        return this;
    }

    /** Requests a repaint of this widget's area. */
    public void repaint() {
        if (container != null) {
            if (invalid) {
                container.repaint();
            } else {
                container.repaint(x,y,w,h);
            }
        }
    }

    /**
     * Sets the position of this widget relative to its
     * container.  The alignment parameter uses the
     * constants for positioning from the MIDP Graphics class.
     */
    public void setPosition(int x, int y, int align) {
        if ((align & Graphics.LEFT) != 0) {
            this.x = x;
        } else if ((align & Graphics.RIGHT) != 0) {
            this.x = x - w;
        } else if ((align & Graphics.HCENTER) != 0) {
            this.x = x - w / 2;
        }
        if ((align & Graphics.TOP) != 0) {
            this.y = y;
        } else if ((align & Graphics.BOTTOM) != 0) {
            this.y = y - h;
        } else if ((align & Graphics.VCENTER) != 0) {
            this.y = y - h / 2;
        }
        invalidate();
        repaint();
    }

    public final void invalidate() {
        if (!invalid) {
            invalid = true;
            if (container != null) {
                container.invalidate();
            }
        }
    }

    public void validate() {
        invalid = false;
    }

    /** 
     * Called when the canvas is first shown.  This implementation
     * does nothing.
     */
    public void showNotify() { }

    /** 
     * Called when the canvas is no longer shown.  This implementation
     * does nothing.
     */
    public void hideNotify() { }
}
