/*
    $Id: BoxContainer.java,v 1.2 2005/11/16 11:51:55 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui.containers;


import com.synclast.Graphics;
import com.synclast.ui.Widget;

/**
 * A container that manages a list of components, arranged
 * either vertically or horizontally.
 */
public class BoxContainer extends Container {
    public static final int HORIZONTAL = Graphics.TOP | Graphics.VCENTER | Graphics.BOTTOM;
    public static final int VERTICAL = Graphics.LEFT | Graphics.HCENTER | Graphics.RIGHT;

    private static final int BACK = Graphics.TOP | Graphics.LEFT;
    private static final int CENTER = Graphics.VCENTER | Graphics.HCENTER;
    private static final int FORWARD = Graphics.BOTTOM | Graphics.RIGHT;
    
    // Workaround for MIDP 1.0 .class access
    public static Class _class = new BoxContainer().getClass();
    private BoxContainer() { }

    private int alignment;
    private int orientation;
    private int nextX, nextY;

    /**
     * Make a box.  Alignment has valid values of:
     * <ul>
     * <li>{@link com.synclast.Graphics LEFT}</li>
     * <li>{@link com.synclast.Graphics HCENTER}</li>
     * <li>{@link com.synclast.Graphics RIGHT}</li>
     * <li>{@link com.synclast.Graphics TOP}</li>
     * <li>{@link com.synclast.Graphics VCENTER}</li>
     * <li>{@link com.synclast.Graphics BOTTOM}</li>
     * <ul>
     * The alignment also determines the orientation of the box.  LEFT, HCENTER, and RIGHT will result in a vertical box, that is a box where the components are stacked on top of each other.  TOP, VCENTER, and BOTTOM will result in a horizontal box, that is a box where the components are layed out strictly left to right.  This seems counter-intuitive at first.  ASCII ART!!!:
     * <pre>
     * LEFT:  
     *   ---------   
     *  | ---     |
     *  || 1 |    |
     *  ||   |    |
     *  | ---     |
     *  | ------- |
     *  || 2     ||
     *  ||       ||
     *  | ------- |
     *  | -----   |
     *  || 3   |  |
     *  ||     |  |
     *  | -----   |
     *   ---------
     *
     * HCENTER:
     *   ---------   
     *  |   ---   |
     *  |  | 1 |  |
     *  |  |   |  |
     *  |   ---   |
     *  | ------- |
     *  || 2     ||
     *  ||       ||
     *  | ------- |
     *  |  -----  |
     *  | | 3   | |
     *  | |     | |
     *  |  -----  |
     *   ---------
     *
     * RIGHT:
     *   ---------   
     *  |     --- |
     *  |    | 1 ||
     *  |    |   ||
     *  |     --- |
     *  | ------- |
     *  || 2     ||
     *  ||       ||
     *  | ------- |
     *  |   ----- |
     *  |  | 3   ||
     *  |  |     ||
     *  |   ----- |
     *   ---------
     * TOP:
     *   ---------------------
     *  | -----  -------  --- |
     *  || 1   || 2     || 3 ||
     *  | ----- |       ||   ||
     *  |       |       ||   ||
     *  |       |       | --- |
     *  |       |       |     |
     *  |        -------      |
     *   ---------------------
     * VCENTER:
     *   ---------------------
     *  |        -------      |
     *  |       | 2     | --- |
     *  | ----- |       || 3 ||
     *  || 1   ||       ||   ||
     *  | ----- |       ||   ||
     *  |       |       | --- |
     *  |        -------      |
     *   ---------------------
     * BOTTOM:
     *   ---------------------
     *  |        -------      |
     *  |       | 2     |     |
     *  |       |       | --- |
     *  |       |       || 3 ||
     *  | ----- |       ||   ||
     *  || 1   ||       ||   ||
     *  | -----  -------  --- |
     *   ---------------------
     *   </pre>
     */
    public BoxContainer(int align) {
        super();
        alignment = calcAlign(align, BACK);
        if((align & VERTICAL) > 0) {
            orientation = VERTICAL;
        } else {
            orientation = HORIZONTAL;
        }
        nextX = 0;
        nextY = 0;
        w = h = 2 * getTotalInset();
    }

    private int calcAlign(int align, int defaultAlign) {
        if((align & BACK) > 0) {
            return BACK;
        } else if((align & CENTER) > 0) {
            return CENTER;
        } else if((align & FORWARD) > 0) {
            return FORWARD;
        } else {
            return defaultAlign;
        }
    }

    private int calcLeadingEdge(int span, int room, int childAlign) {
        int realAlign = alignment;
        if((orientation & childAlign) > 0) {
            realAlign = calcAlign(childAlign, realAlign);
        }
        //realAlign should now only be BACK, FORWARD, or CENTER
        switch(realAlign) {
            case FORWARD:
                return (room - span);
            case CENTER:
                return ((room - span) / 2);
            case BACK:
            default:
                return 0;
        }
    }
    
    public void layout() {
        nextX = 0;
        nextY = 0;
        int size = widgets.size();
        for(int i = 0; i < size; i++) {
            Widget widget = (Widget)widgets.elementAt(i);
            layout(widget);
        }
    }
            
    private void layout(Widget widget) {
        int childAlign = widget.properties[Widget.ALIGN];
        if(orientation == HORIZONTAL) {
            widget.x = nextX;
            nextX += widget.w;
            widget.y = calcLeadingEdge(widget.h, h - 2 * getTotalInset(), childAlign);
        } else { //VERTICAL
            widget.y = nextY;
            nextY += widget.h;
            widget.x = calcLeadingEdge(widget.w, w - 2 * getTotalInset(), childAlign);
        }
    }

    public Container add(Widget w) {
        alterSize(w);
        return super.add(w);
    }
   
    public void remove(Widget widget) {
        super.remove(widget);
        w = h = 2 * getTotalInset();
        Widget localW;
        for(int i = 0; i < widgets.size(); i++) {
           localW = (Widget)widgets.elementAt(i);
           alterSize(localW);
        }
    } 
    
    private void alterSize(Widget widget) {
        if(orientation == HORIZONTAL) {
            w += widget.w;
            h = Math.max(h, widget.h + 2 * getTotalInset());
        } else {
            w = Math.max(w, widget.w + 2 * getTotalInset());
            h += widget.h;
        }
    }
    
    public void validate() {
        //recalc widht and height
        w = h = 2 * getTotalInset();
        Widget widget;
        for(int i = 0; i < widgets.size(); i++) {
            widget = (Widget)widgets.elementAt(i);
            alterSize(widget);
        }
        //layout children
        nextX = 0;
        nextY = 0;
        super.validate();
    }
}
