/*
    $Id: Container.java,v 1.2 2005/11/16 11:51:55 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui.containers;

import java.util.Vector;


import com.synclast.Event;
import com.synclast.Graphics;
import com.synclast.StyleSheet;
import com.synclast.ui.ColoredWidget;
import com.synclast.ui.KeyEventListener;
import com.synclast.ui.PointerEventListener;
import com.synclast.ui.UIContext;
import com.synclast.ui.Widget;

/**
 * A container for widgets.  Containers manage the widgets'
 * focus and event processing.  Each container has its own
 * coordinate system for the widgets it contains.
 * This class implements a container with no layout smarts;
 * all positioning must be done explicitly on the widgets
 * themselves.
 */
public class Container extends ColoredWidget implements KeyEventListener, PointerEventListener {
    // Workaround for MIDP 1.0 .class access
    public static Class _class = new Container().getClass();

    protected Vector widgets = new Vector();
    // Currently focused widget index
    private int focused = -1;
    // Number of focusable widgets
    private int focusable;
    
    public UIContext context;

    /**
     * Creates a new container initially unassociated with any UIContext.
     */
    public Container() { }

    /**
     * Returns the context that is ultimately managing this container.
     */
    public final UIContext getContext() {
        if (context != null) return context;
        if (container == null) return null;
        return container.getContext();
    }

    /**
     * Paints all widgets in the graphics clip area.  The widgets will
     * be painted in the same order they were added to the container;
     * therefore, widgets added later may overlap or cover earlier
     * widgets.  The paint() method is only called on a given widget
     * if the widget overlaps the clipping rectangle. 
     */
    public void paint(Graphics graphics) {
        if (invalid) {
            validate();
        }
        
        super.paint(graphics);
        
        int cx, cy, cw, ch;
		
        // These values will be relative to the coordinate system in use
        cw = graphics.getClipWidth();
        ch = graphics.getClipHeight();
        cx = graphics.getClipX();
        cy = graphics.getClipY();
        
        int oldTX = graphics.getTranslateX();
        int oldTY = graphics.getTranslateY();
        
        Widget widget;
        int size = widgets.size();
        for (int i = 0; i < size; i++) {
            widget = (Widget) widgets.elementAt(i);

            // Check if widget overlaps clip bounds
            graphics.clipRect(widget.x, widget.y, widget.w, widget.h);

            if ((graphics.getClipWidth() > 0)
                && (graphics.getClipHeight() > 0)) {
                graphics.translate(widget.x, widget.y);
                widget.paint(graphics);
                graphics.translate(oldTX - graphics.getTranslateX(), 
                                   oldTY - graphics.getTranslateY());
            }
            // Reset the clip bounds
            graphics.setClip(cx, cy, cw, ch);
        }
    }

    /** Repaints this container. */
    public void repaint() {
        repaint(x,y,w,h);
    }

    /**
     * Requests a repaint for the specified region (in the coordinate
     * system of this container).
     */
    public void repaint(int rx, int ry, int rw, int rh) {
        if (context != null) {
            if (invalid) {
                context.repaint();
            } else {
                context.repaint(rx, ry, rw, rh);
            }
        } else {
            // Translate and invoke on parent container
            if (container != null) {
                if(invalid) {
                    container.repaint();
                } else {
                    container.repaint(x + rx, y + ry, rw, rh);
                }
            }
        }
    }

    /**
     * Applies the given style sheet to this container and its
     * widgets, recursively.
     * At the end of each application of
     * a style sheet, the invalidate() method is called, because some
     * components may have been resized.
     */
    public void applyStyleSheet(StyleSheet sheet) {
        super.applyStyleSheet(sheet);

        // Apply just the new styleSheet to contained widgets
        Widget widget;
        for (int i = widgets.size() - 1; i >= 0; --i) {
            widget = (Widget) widgets.elementAt(i);
            widget.applyStyleSheet(sheet);
        }
    }

    /**
     * Informs the container that initialization is complete and it may
     * lay out its widgets.  layout() is invoked whenever a new style sheet
     * is applied.  It will be invoked at least once before a Container
     * is painted.
     */
    protected void layout() {
        // Recalculate width and height
        w = h = 0;
        for (int i = widgets.size() - 1; i >= 0; --i) {
            Widget widget = (Widget) widgets.elementAt(i);
            w = Math.max(w, widget.x + widget.w );
            h = Math.max(h, widget.y + widget.h);
        }
        int offset = 2 * getTotalInset();
        w += offset;
        h += offset;
    }

    /**
     * Adds a new widget to the container.
     * The first action taken is to apply the current style sheet
     * to the widget.  Next, the widget has its setContainer method
     * called.  If the container is not focusable, and this widget is, 
     * focus is transfered.  Then the container is invalidated, and if visble,
     * repainted.  Note:  if the widget is already in the container,
     * everything is still done the same way, except that the widget is not
     * re-added to the list of children, and therefore, will not change its
     * order in subclasses that are order dependent.  If this is desired, do a
     * remove(widget), add(widget).
     * @param widget the Widget to add to this container.
     * @return convenience this for doing multiple adds.
     */
    public Container add(Widget widget) {
        // Apply style sheet, if available
        UIContext context = getContext();
        if (context != null) {
            widget.applyStyleSheet(context.getStyleSheet());
        }
        widget.setContainer(this);
        if(!widgets.contains(widget)) {
            widgets.addElement(widget);
        }
        if (context != null && context.isShown()) {
            widget.showNotify();
        }
        // Update focusable status of the container as a whole
        if (widget instanceof KeyEventListener) {
            setFocusable(1);
        }
        invalidate();
        return this;
    }

    /**
     * Removes the specified widget from the container.
     * This call forces a repaint of the container.
     *
     * @param widget the widget to remove.  If the widget is currently
     * not added to the container, no action is taken.
     */
    public void remove(Widget widget) {
        if(widgets.removeElement(widget)) {
            UIContext context = getContext();
            if (context != null && context.isShown()) {
                widget.hideNotify();
            }
            widget.setContainer(null);
            if (widget instanceof KeyEventListener) {
                setFocusable(-1);
            }
            invalidate();
            repaint();
        }
    }

    /**
     * Dispatches a keyReleased event.  If there is a currently
     * focused widget, that widget is given a chance to handle the event.
     */
    public void keyReleased(int keyCode) {
        if (focused != -1) {
            ((KeyEventListener) widgets.elementAt(focused)).keyReleased(keyCode);
        }
    }

    /**
     * Dispatches a keyRepeated event.  If there is a currently
     * focused widget, that widget is given a chance to handle the event.
     */
    public void keyRepeated(int keyCode) {
        if (focused != -1) {
            ((KeyEventListener) widgets.elementAt(focused)).keyRepeated(keyCode);
        }
    }

    /**
     * Dispatches a keyPressed event.  If there is a currently
     * focused widget, that widget is given a chance to handle the event.
     */
    public void keyPressed(int keyCode) {
        if (focused != -1) {
            ((KeyEventListener) widgets.elementAt(focused)).keyPressed(keyCode);
        }
    }

    /**
     * Dispatches a pointerPressed event.
     * All widgets that implement the PointerEventListener interface
     * have their pointerPressed method called until one returns true.
     */
    public boolean pointerPressed(int x, int y) {
        for (int i = 0; i < widgets.size(); i++) {
            Widget widget = (Widget) widgets.elementAt(i);
            if (widget instanceof PointerEventListener) {
                if (((PointerEventListener) widget).pointerPressed(x, y)) {
                    repaint();
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Dispatches a pointerDragged event.
     * All widgets that implement the PointerEventListener interface
     * have their pointerDragged method called until one returns true.
     */
    public boolean pointerDragged(int x, int y) {
        for (int i = 0; i < widgets.size(); i++) {
            Widget widget = (Widget) widgets.elementAt(i);
            if (widget instanceof PointerEventListener) {
                if (((PointerEventListener) widget).pointerDragged(x, y)) {
                    repaint();
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Dispatches a pointerReleased event.
     * All widgets that implement the PointerEventListener interface
     * have their pointerReleased method called until one returns true.
     */
    public boolean pointerReleased(int x, int y) {
        for (int i = 0; i < widgets.size(); i++) {
            Widget widget = (Widget) widgets.elementAt(i);
            if (widget instanceof PointerEventListener) {
                if (((PointerEventListener) widget).pointerReleased(x, y)) {
                    repaint();
                    return true;
                }
            }
        }
        return false;
    }

    public void focus() {
        if (focusable != 0) {
            passFocus(1);
        }
    }

    public void blur() {
        if (focused != -1) {
            ((KeyEventListener) widgets.elementAt(focused)).blur();
            focused = -1;
        }
    }

    private void setFocusable(int delta) {
        this.focusable += delta;
        if ((container != null)
            && (((delta == -1) && (focusable == 0)) 
                || ((delta == 1) && (focusable == 1)))) {
            container.setFocusable(delta);
        }
    }

    /**
     * Provides a mechanism for one widget to move the focus to
     * the next or previous widget.  The order of focus corresponds
     * to the order in which focusable widgets were added to the
     * container.  To indicate the previous widget, pass -1; for the next,
     * pass 1.  Widgets that wish to simply unfocus should just call
     * their blur() command; they will still be responsible for passing
     * the focus under the appropriate circumstances.
     * 
     * The result of this method is that a previously focused widget
     * will have its blur() method called, and the newly focused widget
     * will have its focus() method called.
     */
    public void passFocus(int direction) {
        if (focusable == 0) {
            if (container != null) {
                container.passFocus(direction);
            }
            return;
        }

        if (focused != -1) {
            ((KeyEventListener) widgets.elementAt(focused)).blur();
        } else if (direction < 0) {
            focused = widgets.size();
        }

        int size = widgets.size();
        // Find the new widget to focus on
        while (true) {
            focused += direction;
            if (focused < 0 || focused >= size) {
                if (container != null) {
                    focused = -1;
                    container.passFocus(direction);
                    return;
                } else {
                    focused += size; // ensure not negative
                    focused %= size;
                }
            }

            Widget widget = (Widget) widgets.elementAt(focused);
            if (widget instanceof Container) {
                if (((Container) widget).focusable > 0) {
                    ((Container) widget).passFocus(direction);
                }
                return;
            } else if (widget instanceof KeyEventListener) {
                ((KeyEventListener) widget).focus();
                return;
            }
        }
    }

    /**
     * Requests to gain the focus.  The widget's focus() method
     * will be called after the previously focused widget's blur()
     * call.  If the widget is already in focus, nothing happens.
     */
    public void requestFocus(KeyEventListener widget) {
        int i = widgets.indexOf(widget);
        if (i != focused) {
            if (focused != -1) {
                ((KeyEventListener) widgets.elementAt(focused)).blur();
            }
            focused = i;
            widget.focus();
        }
    }

    /**
     * Validate the container.  This consists of walking children, and
     * for each, validating them if invalid, and then laying them out
     * in the container space.  The container will be valid after this
     * call.  Subclasses wishing to override this method (such as to
     * change the order of layout calls) should make sure that all
     * invalid children have validate() called on them and that
     * invalid is set to false on the container before returning from
     * this method.  Children should be validated before being layed
     * out, as they might change their size when they are validated.  
     */
    public void validate() {
        int size = widgets.size();
        for(int i = 0; i < size; i++) {
            Widget w = (Widget)widgets.elementAt(i);
            if(w.invalid) {
                w.validate();
            }
        }
        layout();
        invalid = false;
    }

    /** Propagates showNotify() call to all contained widgets. */
    public void showNotify() {
        int size = widgets.size();
        for(int i = 0; i < size; i++) {
            ((Widget) widgets.elementAt(i)).showNotify();
        }
    }

    /** Propagates hideNotify() call to all contained widgets. */
    public void hideNotify() {
        int size = widgets.size();
        for(int i = 0; i < size; i++) {
            ((Widget) widgets.elementAt(i)).hideNotify();
        }
    }

    /**
     * Overrides fireEvent on widget to offload to the UIContext's
     * CommandListener if none is set at this level and this is
     * the top-level container.
     */
    public boolean fireEvent(Event event) {
        return super.fireEvent(event) || 
            ((context != null) && context.fireEvent(event));
    }
}
