/*
    $Id: FlowContainer.java,v 1.1.1.1 2005/11/14 17:33:29 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui.containers;


import com.synclast.Graphics;
import com.synclast.ui.ColoredWidget;
import com.synclast.ui.Widget;

/**
 * A container that automatically positions components
 * in horizontal or vertical rows.  (For the sake of convenience,
 * we use the term "row" loosely here.)
 */
public class FlowContainer extends Container {
    private static final Widget ROW_BREAK = new ColoredWidget();

    public static final int HORIZONTAL =
        Graphics.TOP | Graphics.VCENTER | Graphics.BOTTOM;
    public static final int VERTICAL = 
        Graphics.LEFT | Graphics.HCENTER | Graphics.RIGHT;

    // Workaround for MIDP 1.0 .class access
    public static Class _class = new FlowContainer(0).getClass();

    private int align;

    /**
     * Creates a new FlowContainer with the specified alignment,
     * which should be one of the alignment constants from the 
     * LCDUI Graphics class.  TOP, VCENTER and BOTTOM will create
     * a horizontally flowing layout (see also BoxContainer).
     * LEFT, HCENTER and RIGHT will create a vertically flowing layout.
     * As items are added to the container, they will be positioned
     * logically following the previous item (flowing either left
     * to right or top to bottom, depending on the alignment specified).
     */
    public FlowContainer(int align) {
        this.align = align;
    }

    /**
     * Invoking this method indicates that the next widget
     * added should be positioned on a new row.  Note that with a
     * vertical alignment setting, this may be more appropriately
     * thought of as "nextColumn()".
     */
    public FlowContainer nextRow() {
        widgets.addElement(ROW_BREAK);
        return this;
    }

    protected void layout() {
        boolean vertical = (VERTICAL & align) != 0;
        // The x/y names make sense if we're talking about horizontal
        // layout, but for vertical they end up getting swapped.

        int nextX = 0, nextY = 0, rowHeight = 0;

        // Calculate w, h; assign x, y to contained widgets
        for (int i = 0; i < widgets.size(); i++) {
            Widget widget = (Widget) widgets.elementAt(i);

            if (widget == ROW_BREAK) {
                nextX = 0;
                nextY += rowHeight;
                rowHeight = 0;
            } else {
                // Position relative to last one.
                if (vertical) {
                    widget.y = nextX;
                    widget.x = nextY;
                    if (widget.w > rowHeight) {
                        rowHeight = widget.w;
                    }
                    if (nextX > h) {
                        h = nextX;
                    }
	    
                    nextX = widget.y + widget.h;
                    if (nextY > w) {
                        w = nextY;
                    }
                } else {
                    widget.x = nextX;
                    widget.y = nextY;
                    if (widget.h > rowHeight) {
                        rowHeight = widget.h;
                    }
                    if (nextY > h) {
                        h = nextY;
                    }
	    
                    nextX = widget.x + widget.w;
                    if (nextX > w) {
                        w = nextX;
                    }
                }
            }
        }
    }
}
