/*
    $Id: TableContainer.java,v 1.2 2005/11/16 11:51:55 val Exp $

    This file is part of the Synclast UI API.

    The Synclast UI API is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    The Synclast UI API is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the Synclast UI API; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.synclast.ui.containers;

import java.util.Vector;

import com.synclast.ui.ColoredWidget;
import com.synclast.ui.Widget;

/**
 * This class is modeled after an HTML table.  Each row must be explicitly
 * ended, and the table dynamically resizes to fit the content specified.
 */
public class TableContainer extends Container {
    // Workaround for MIDP 1.0 .class access
    public static Class _class = new TableContainer().getClass();

    private static final Widget RESERVED_SPACE = new ColoredWidget();

    private int rows;
    private int cols;
    private int xPos;
    private boolean fixedSize;

    // These are per-widget, matching index in widgets vector
    private Vector colspans = new Vector();
    private Vector rowspans = new Vector();

    // These are only updated when validate() is called
    private Vector widths = new Vector(); // one per column
    private Vector heights = new Vector(); // one per row

    private Widget[][] contents;

    /**
     * Creates a new TableContainer with a dynamic number of rows
     * and columns.  The nextRow() method should be called to
     * begin a new row.
     */
    public TableContainer() { }

    /**
     * Creates a new TableContainer with a fixed number of rows
     * and columns.  Tables generated in this manner do not need
     * to have the nextRow() method invoked.
     */
    public TableContainer(int cols, int rows) {
        fixedSize = true;
        widths.setSize(cols);
        heights.setSize(rows);
        this.cols = cols;
        this.rows = rows;
    }

    /** Returns the number of columns in the table. */
    public int getColumns() {
        return cols;
    }

    /** Returns the number of rows in the table.  
     * For dynamically sized tables, this value will be 0
     * until the first call to nextRow().
     */
    public int getRows() {
        return rows;
    }

    /** Adds the widget in the next available position in the table. */
    public Container add(Widget widget) {
        return add(widget, 1, 1);
    }

    /** Adds the widget with a specified column width and row height. */
    public TableContainer add(Widget widget, int colspan, int rowspan) {
        if (!fixedSize) {
            xPos += colspan;
            if (xPos > cols) {
                cols = xPos;
            }
        }

        colspans.addElement(new Integer(colspan));
        rowspans.addElement(new Integer(rowspan));
        super.add(widget);
        return this;
    }

    public TableContainer nextRow() {
        ++rows;
        if (xPos > cols) {
            cols = xPos;
        }
        xPos = 0;
        return this;
    }

    protected void layout() {
        if (!fixedSize) {
            widths.setSize(cols);
            heights.setSize(rows);
        }

        Widget widget;
        // Build contents array
        contents = new Widget[cols][rows];

        int i = 0;

        // Assign widgets or RESERVED_SPACE to each cell of contents
	yLoop: 
        for (int y = 0; y < rows; y++) {
            for (int x = 0; x < cols; x++) {
                if (contents[x][y] == null) {
                    // Assign the next widget here
                    widget = (Widget) widgets.elementAt(i);
                    int cspan = ((Integer) colspans.elementAt(i)).intValue();
                    int rspan = ((Integer) rowspans.elementAt(i)).intValue();
                    if ((cspan > 1) || (rspan > 1)) {
                        for (int cs = 0; cs < cspan; cs++) {
                            for (int rs = 0; rs < rspan; rs++) {
                                contents[x + cs][y + rs] = RESERVED_SPACE;
                            }
                        }
                    }
                    contents[x][y] = widget;
                    if (++i >= widgets.size()) { break yLoop; }
                }
            }
        }

        // The height of each row is the max of any widget set for the row
        // The width of each column is the max of any widget set for the column
        // The width of all rows is the sum(column widths)
        // Total height = sum(row heights)
        // Total width = max(row widths)

        int[] columnWidths = new int[cols];
        for (int x = 0; x < cols; x++) {
            for (int y = 0; y < rows; y++) {
                widget = contents[x][y];
                if ((widget != null) && (widget != RESERVED_SPACE)) {
                    int cspan = ((Integer) colspans.elementAt(widgets.indexOf(widget))).intValue();
                    if (cspan == 1) {
                        if (widget.w > columnWidths[x]) {
                            columnWidths[x] = widget.w;
                        }
                    }
                }
            }
        }

        int[] rowHeights = new int[rows];
        for (int y = 0; y < rows; y++) {
            for (int x = 0; x < cols; x++) {
                widget = contents[x][y];
                if ((widget != null) && (widget != RESERVED_SPACE)) {
                    int rspan = ((Integer) rowspans.elementAt(widgets.indexOf(widget))).intValue();
                    if (rspan == 1) {
                        if (widget.h > rowHeights[y]) {
                            rowHeights[y] = widget.h;
                        }
                    }
                }
            }
        }

        // Determine total width and height
        // TODO allow borders, spacing, etc.
        // This part turns columnWidths/columnHeights into absolute positions (running totals).
        w = h = 2 * getTotalInset();
        for (int x = 1; x < cols; x++) {
            columnWidths[x] = columnWidths[x - 1] + columnWidths[x];
        }
        w += columnWidths[cols - 1];
        for (int y = 1; y < rows; y++) {
            rowHeights[y] = rowHeights[y - 1] + rowHeights[y];
        }
        h += rowHeights[rows - 1];

        // Now set x/y positions of all widgets
        for (int x = 0; x < cols; x++) {
            for (int y = 0; y < rows; y++) {
                widget = contents[x][y];
                if ((widget != null) && (widget != RESERVED_SPACE)) {
                    // Currently these assume TOP | LEFT alignment
                    widget.x = (x > 0) ? columnWidths[x - 1] : 0;
                    widget.y = (y > 0) ? rowHeights[y - 1] : 0;
                }
            }
        }
        
    }
}
