package val.games.paperotte;

import java.util.Vector;

import sdljava.SDLException;
import sdljava.video.SDLColor;
import sdljava.video.SDLSurface;
import val.games.paperotte.functions.*;
import val.gp2x.GP2x;


public class ALP_GameLevel implements GamePhase
{
  private GP2x gp2x;
  private ALP_SharedGameData sharedData;

  private Vector activeSeries;
  private Mirino mirino;

  private long lastFlap = 0;
  private boolean finished = false;

  private int exitCode = GAMEPHASE_ERROR;

  // configure some part only once
  private boolean configured = false;
  
  // ************************ Configuration *************************************
  public int configure(GP2x gp2x, GameData gameData)
  {
    this.gp2x = gp2x;
    this.sharedData = (ALP_SharedGameData)gameData;

    try
    {
      // Now load
      if (!configured)
      {
        activeSeries = new Vector(1);
        mirino = new Mirino(new Sprite(gp2x.getFramebuffer(), 1, sharedData.spriteMirino), sharedData.fireSound);
        mirino.setInitialPosition(gp2x.getFramebuffer().getWidth() / 2, gp2x.getFramebuffer().getHeight() / 2);
  
        lastFlap = System.currentTimeMillis();
        
        configured = true;
      }
      
      setupLevel();

      finished = false;
      exitCode = GAMEPHASE_ERROR;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return GAMEPHASE_ERROR;
    }
    
    return GAMEPHASE_OK;
  }

  private void setupLevel()
  {
    if (sharedData.gameType == ALP_SharedGameData.GAME_MODE_STORY)
    {
      switch (sharedData.gameLevel)
      {
        // First 4, only one wave
        case 1: createPapereSerie(0); break;
        case 2: createPapereSerie(1); break;
        case 3: createPapereSerie(2); break;
        case 4: createPapereSerie(3); break;

        // Second 4, two waves
        case 5:
          createPapereSerie(0);
          createPapereSerie(1);
        break;
        case 6:
          createPapereSerie(1);
          createPapereSerie(2);
        break;
        case 7:
          createPapereSerie(2);
          createPapereSerie(3);
        break;
        case 8:
          createPapereSerie(3);
          createPapereSerie(0);
        break;
        
        default:
          // game finished, switch to Infinity mode
          GamePhase gameCongrats = new ALP_GameCongrats();
          if (gameCongrats.configure(gp2x, sharedData) != GamePhase.GAMEPHASE_ERROR)
          {
            gameCongrats.mainloop();
            gameCongrats.finish();
            // congratulation screen can only return ok
          }

          sharedData.gameType = ALP_SharedGameData.GAME_MODE_INFINITY;
          sharedData.gameLevel = 1;
        break;
      }
    }
    
    if (sharedData.gameType == ALP_SharedGameData.GAME_MODE_INFINITY)
    {
      // Create three series for infinity mode
      createRandomPapereSerie();
      createRandomPapereSerie();
      createRandomPapereSerie();
    }
  }
  
  
  public int prepare_resume()
  {
    finished = false;
    return GAMEPHASE_OK;
  }
  
  // ************************ Main Loop *************************************
  public int mainloop()
  {
    try
    {
      int frames = 0;
      long start = System.currentTimeMillis();
      while (!finished)
      {
        Thread.sleep(100); // We repaint the screen every 33 micro seconds
        events();
        sounds();
        update();
        repaint();
        frames++;
      }
      long end = System.currentTimeMillis();

      if (end > start)
      {
	      System.out.println("FPS: start="+start+" end="+end);
	      int fps = (int)(frames / ((end-start)/1000.0));
	      System.out.println("FPS: "+frames+" in "+(end-start)+" msecs: "+fps);
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      return GAMEPHASE_ERROR;
    }

    return exitCode;
  }

  
  // ************************ Finalization *************************************
  public int finish()
  {
    activeSeries.removeAllElements();
    activeSeries = null;
    mirino = null;
    
    return GAMEPHASE_OK;
  }


  private void createPapereSerie(int index)
  {
    SerieOfPaperotte sop = null;
    SDLSurface type = null;
    Function f = null;
    
    switch ( sharedData.random.nextInt(3) )
    {
      case 0:        type = sharedData.spritePaperaBlu;        break;
      case 1:        type = sharedData.spritePaperaRossa;        break;
      case 2:        type = sharedData.spritePaperaVerde;        break;
    }
    
    switch ( index )
    {
      case 0:        f = new SinWave();        break;
      case 1:        f = new CosWave();        break;
      case 2:        f = new SinCosWave();        break;
      case 3:        f = new QuadWave();        break;
    }
    
    int number = 8 + sharedData.random.nextInt(9); // 8..16
    int offset = 30 + sharedData.random.nextInt(31); // 30..60
    int speed = 2 + sharedData.random.nextInt(5); // 2..6
    
    // Segna le nuove papere
    sharedData.paperePassate += number;
    
    sop = new SerieOfPaperotte(gp2x, number, offset, speed, type, f);
    activeSeries.addElement(sop);
  }

  private void createRandomPapereSerie()
  {
    int f = sharedData.random.nextInt(4);
    createPapereSerie(f);
  }

  
  private void sounds()
  {
    if (System.currentTimeMillis() - lastFlap > 1000)
    {
      sharedData.flapSound.playTwice();
      lastFlap = System.currentTimeMillis();
    }
  }

  public void events() throws Exception
  {
    int speed = 5;
    
    gp2x.getJoyButtonChanges();

    if (gp2x.isButtonPressed(GP2x.VK_UP)) {
      mirino.moveTo(0, -speed);
    }
    if (gp2x.isButtonPressed(GP2x.VK_DOWN)) {
      mirino.moveTo(0, speed);
    }
    if (gp2x.isButtonPressed(GP2x.VK_LEFT)) {
      mirino.moveTo(-speed, 0);
    }
    if (gp2x.isButtonPressed(GP2x.VK_RIGHT)) {
      mirino.moveTo(speed, 0);
    }

    if (gp2x.isButtonPressed(GP2x.VK_UP_LEFT)) {
      mirino.moveTo(-speed, -speed);
    }
    if (gp2x.isButtonPressed(GP2x.VK_UP_RIGHT)) {
      mirino.moveTo(speed, -speed);
    }
    if (gp2x.isButtonPressed(GP2x.VK_DOWN_LEFT)) {
      mirino.moveTo(-speed, speed);
    }
    if (gp2x.isButtonPressed(GP2x.VK_DOWN_RIGHT)) {
      mirino.moveTo(speed, speed);
    }

    if (gp2x.isButtonClicked(GP2x.VK_FX)) {
      sharedData.colpiSparati++;
      mirino.fireSound.playOnce();
      
      for (int i = 0; i < activeSeries.size(); i++)
      {
        SerieOfPaperotte serie = (SerieOfPaperotte) activeSeries.elementAt(i);
        Paperotta p = null;
        while ((p = serie.isHit(mirino.xCenter, mirino.yCenter)) != null)
        {
          // segna un punto e fai Quack !
          sharedData.papereAmmazzate++;
          sharedData.quacksSound.playOnce();

          p.isMorta = true;
          p.function = new SequenzaDiMorte();
          Sprite s = p.sprite;
          
          if (serie.getSpriteSurface() == sharedData.spritePaperaBlu)
            p.sprite = new Sprite(gp2x.getFramebuffer(), 2, sharedData.spritePaperaBluMorta);
          else if (serie.getSpriteSurface() == sharedData.spritePaperaRossa)
            p.sprite = new Sprite(gp2x.getFramebuffer(), 2, sharedData.spritePaperaRossaMorta);
          else if (serie.getSpriteSurface() == sharedData.spritePaperaVerde)
            p.sprite = new Sprite(gp2x.getFramebuffer(), 2, sharedData.spritePaperaVerdeMorta);
          
          p.setInitialPosition(s.position.x, s.position.y);
        }
      }

    }
    if (gp2x.isButtonClicked(GP2x.VK_START)) {
      finished = true;
      exitCode = GAMEPHASE_PAUSE;
    }
        
    if (gp2x.isButtonClicked(GP2x.VK_SELECT)) {
      finished = true;
      exitCode = GAMEPHASE_QUIT;
    }
        
  }
  
  
  public void repaint()
  {
    try
    {
      gp2x.drawImage(sharedData.game_background, 0, 0);

      String s = "";
      if (sharedData.gameType == ALP_SharedGameData.GAME_MODE_STORY)
        s = "Story Livello "+sharedData.gameLevel;
      else
        s = "Onde Infinite";
      
      s = s + "  Papere "+sharedData.papereAmmazzate+"/"+sharedData.paperePassate;
      s = s + "  Colpi "+sharedData.colpiSparati;
      
      SDLSurface s1 = sharedData.gameFont.writeString(s, new SDLColor(255, 0, 0));
      gp2x.drawImage(s1, 5, 5);
      s1.freeSurface();
  
      for (int i = 0; i < activeSeries.size(); i++)
      {
        SerieOfPaperotte sop = (SerieOfPaperotte) activeSeries.elementAt(i);
        sop.redraw();
      }
      
      mirino.redraw();
      
      gp2x.repaint();
    }
    catch (SDLException e)
    {
      e.printStackTrace();
    }
  }

  
  public void update()
  {
    for (int i = activeSeries.size() - 1; i >= 0; i--)
    {
      if (! ((SerieOfPaperotte) activeSeries.elementAt(i)).hasElementsInGame())
        activeSeries.remove(i);
    }
    
    if (activeSeries.size() == 0)
    {
      // finished this level, if in Story mode, advance the level !
      finished = true;
      exitCode = GAMEPHASE_OK;

      if (sharedData.gameType == ALP_SharedGameData.GAME_MODE_STORY)
        sharedData.gameLevel++;
    }
    else
    {
      for (int i = 0; i < activeSeries.size(); i++)
      {
        ((SerieOfPaperotte) activeSeries.elementAt(i)).move();
      }
    }

  }

}
